#!/bin/bash
#
# This shell script is needed to correctly compile pgplot-related programs on stupid Ubuntu where,
# by some strange reason, pgplot-related programs segfault if compiled directly using make...
#
# This script is intended to be started automatically from the Makefile
#


VAST_DIR=$PWD
TARGET_DIR=$VAST_DIR/lib
LIBRARY_SOURCE=$VAST_DIR/src/cfitsio

echo " "
echo -e "Starting script \033[01;32m$0 $1\033[00m"


if [ "$1" == "clean" ];then
 echo -e "\033[01;34mRemoving the local copy of CFITSIO library\033[00m"
 cd $LIBRARY_SOURCE
 make clean
 make distclean
 rm -f *.pdf *.ps
 echo "Script $0 is done."
 echo " "
 exit
fi


echo -e "\033[01;34mCompiling CFITSIO library\033[00m"
echo "Using C compiler: $C_COMPILER" 


cd $LIBRARY_SOURCE
make clean
./configure --prefix=$TARGET_DIR
make -j4
make fitscopy
cp fitscopy $VAST_DIR/util
cp libcfitsio.a $TARGET_DIR/libcfitsio.a
cp fitsio.h longnam.h $VAST_DIR/src
make clean
cd $VAST_DIR

# Test if executable files were actually created?
COMPILATION_ERROR=0
echo -n "Checking library files:   "
for TEST_FILE in $TARGET_DIR/libcfitsio.a ;do
 echo -n "$TEST_FILE - "
 if [ ! -f $TEST_FILE ];then
  COMPILATION_ERROR=1
  echo -ne "\033[01;31mERROR\033[00m,   "
 else
  echo -ne "\033[01;32mOk\033[00m,   "
 fi
done
echo "done!"

if [ $COMPILATION_ERROR -eq 1 ];then
 echo -e "\033[01;31mCOMPILATION ERROR\033[00m"
 exit 1
fi

echo -e "\033[01;34mFinished compiling CFITSIO library\033[00m"
echo " "
