#!/bin/bash
#
# This shell script is needed to correctly compile pgplot-related programs on stupid Ubuntu where,
# for some strange reason, pgplot-related programs segfault if compiled directly using make...
#
# This script is intended to be started automatically from the Makefile
#


# Check if C compiler name was supplied
if [ ! -z $1 ];then
 C_COMPILER=$1
else
 C_COMPILER=gcc
fi
# Check if location of CFITSIO lib was specified
if [ ! -z $2 ];then
 CFITSIO_LIB=$2 
else
 CFITSIO_LIB=lib/libcfitsio.a
fi
# Check if location of X11 libs were specified
if [ ! -z $3 ];then
 X11_LIB=$3 
else
 X11_LIB=/usr/X11R6/lib
fi

# Get info about libpng
LIBPNG=`lib/test_libpng.sh`

# Compile PGPLOT library
cd lib/pgplot/
make
make cpg 
make clean
cd -

# Compile VaST programs that use PGPLOT lib.
PGPLOT_LIBS="lib/pgplot/libcpgplot.a lib/pgplot/libpgplot.a -L$X11_LIB -lX11 -lgcc `lib/find_fortran_library.sh` $LIBPNG"
GSL_LIB="lib/lib/libgsl.a lib/lib/libgslcblas.a"
GSL_INCLUDE="lib/include"

echo " "
echo -e "Starting script \033[01;32m$0\033[00m"
echo -e "\033[01;34mCompiling PGPLOT-related components\033[00m"
echo "Using C compiler: $C_COMPILER" 
echo "Assuming X11 libraries are located in $X11_LIB"
echo "Libraries needed to compile C PGPLOT programs: $PGPLOT_LIBS"

#"$C_COMPILER" -o lc src/lc.c -lm -lgsl -lgslcblas $PGPLOT_LIBS  -Wall
"$C_COMPILER" -c src/lc.c -I$GSL_INCLUDE
"$C_COMPILER" -o lc setenv_local_pgplot.o lc.o -lm $GSL_LIB $PGPLOT_LIBS  -Wall
#"$C_COMPILER" -o find_candidates src/find_candidates.c -lm $PGPLOT_LIBS
"$C_COMPILER" -c src/find_candidates.c -I$GSL_INCLUDE
"$C_COMPILER" -o find_candidates setenv_local_pgplot.o find_candidates.o $PGPLOT_LIBS -lm
"$C_COMPILER" -c -o photocurve.o src/photocurve.c -I$GSL_INCLUDE -Wall
"$C_COMPILER" -c src/pgfv/pgfv.c -I$GSL_INCLUDE
"$C_COMPILER" -o pgfv pgfv.o setenv_local_pgplot.o photocurve.o gettime.o autodetect_aperture.o exclude_region.o $CFITSIO_LIB -lm $GSL_LIB $PGPLOT_LIBS
"$C_COMPILER" -c -o fit_mag_calib.o src/fit_mag_calib.c -I$GSL_INCLUDE
"$C_COMPILER" -o lib/fit_mag_calib fit_mag_calib.o setenv_local_pgplot.o wpolyfit.o photocurve.o $PGPLOT_LIBS -lm $GSL_LIB -Wall

# Test if executable files were actually created?
COMPILATION_ERROR=0
echo -n "Checking executable files:   "
for TEST_FILE in lib/pgplot/libpgplot.a lib/pgplot/libcpgplot.a lib/pgplot/grfont.dat lc find_candidates pgfv lib/fit_mag_calib ;do
 echo -n "$TEST_FILE - "
 if [ ! -f $TEST_FILE ];then
  COMPILATION_ERROR=1
  echo -ne "\033[01;31mERROR\033[00m,   "
 else
  echo -ne "\033[01;32mOk\033[00m,   "
 fi
done
echo "done!"

if [ $COMPILATION_ERROR -eq 1 ];then
 echo -e "\033[01;31mCOMPILATION ERROR\033[00m"
 exit 1
fi

echo -e "\033[01;34mFinished compiling PGPLOT-related components\033[00m"
echo " "
