#!/bin/bash 
#################################
# This script will upload a lightcurve to the web-based period search service.
#################################
# Check if lightcurve file was supplied ($1)
if [ -z "$1" ];then
 echo "ERROR: no lightcurve file name is supplied!"
 echo " "
 echo "Usage:  $0 outNNNN.dat"
 exit
fi
#################################
# Check if lightcurve file was ($1) exist
if [ ! -f "$1" ];then
 echo "ERROR: lightcurve file $1 does not exist!"
 echo " "
 echo "Usage:  $0 outNNNN.dat"
 exit
fi
#################################
# Find a web browser
WEBBROWSER="none"
if which firefox &>/dev/null ;then
 WEBBROWSER="firefox"
elif which firefox-bin &>/dev/null ;then
 WEBBROWSER="firefox-bin"
elif which chromium &>/dev/null ;then
 WEBBROWSER="chromium"
elif which midori &>/dev/null ;then
 WEBBROWSER="midori"
fi
if [ "$WEBBROWSER" == "none" ];then
 echo "ERROR cannot find a known webbrowser in $PATH"
 echo "Please edit the script $0 and specify a browser manually!"
 exit 1
fi
#################################
# Thest if curl is installed
CURL=`which curl`
if [ $? -ne 0 ];then
 echo " "
 echo "ERROR: curl not found. :("
 echo "Please install curl ..."
 echo " "
 exit 1
fi
#################################
# Do the actual work

# Make up a good name for the converted file
NEWFILENAME=`basename "$1"`
NEWFILENAME=${NEWFILENAME// /_}

# Convert the input lightcurve file to the software-friendly ASCII
lib/formater_out_wfk "$1" > saved_period_search_lightcurves/"$NEWFILENAME" #lightcurve$$.tmp

# Make sure we have a directory to save the converted file
if [ ! -d saved_period_search_lightcurves ];then
 mkdir saved_period_search_lightcurves
fi

# And save the file
echo "# "
echo "# Saving lightcurve file to saved_period_search_lightcurves/"$NEWFILENAME" for possible future use..."
#echo "cp lightcurve$$.tmp saved_period_search_lightcurves/"$NEWFILENAME""
echo "# "
echo "# "
#cp lightcurve$$.tmp saved_period_search_lightcurves/"$NEWFILENAME"

# Determine good period search range
JD_MIN=`sort -n saved_period_search_lightcurves/$NEWFILENAME |head -n1 | awk '{print $1}'`
JD_MAX=`sort -n saved_period_search_lightcurves/$NEWFILENAME |tail -n1 | awk '{print $1}'`
PMAX=`echo "
define abs(i) {
 if (i < 0) return (-i)
 return (i)
}
jd_range=abs(($JD_MAX)-($JD_MIN))
define max_period(jd_range) {
 if( jd_range > 1000 ) return (100)
 if( jd_range > 100 ) return (10)
 if( jd_range > 10 ) return (1)
 return (0.3)
}
max_period(jd_range)
"|bc -ql`

if [ "$PMAX" == ".3" ];then
 PMIN=0.03
else
 PMIN=0.1
fi

# Upload the converted lightcurve file to the server
curl -F file=@"saved_period_search_lightcurves/$NEWFILENAME" -F submit="Compute" -F pmax=$PMAX -F pmin=$PMIN -F phaseshift=0.05 -F fileupload="True" -F applyhelcor="No" -F timesys=`cat vast_summary.log | grep "JD time system" |awk '{print $5}'` -F position="00:00:00.00 +00:00:00.0" "http://scan.sai.msu.ru/cgi-bin/lk/process_lightcurve.py" --user vast48:khyzbaojMhztNkWd > server_reply$$.html

# Parse the server reply
RESULTURL=`grep "The output will be written to" server_reply$$.html |awk -F"<a" '{print $2}' |awk -F">" '{print $1}'`
RESULTURL=${RESULTURL//\"/ }
RESULTURL=`echo $RESULTURL | awk '{print $2}'`

# Remove temporary files
#rm -f lightcurve$$.tmp
rm -f server_reply$$.html

# Start web browser
echo "# Starting $WEBBROWSER web browser..."
echo "$WEBBROWSER $RESULTURL"
$WEBBROWSER $RESULTURL
