#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define VSX_SEARCH_RADIUS_DEG 120.0/3600.0

/* Auxiliary definitions */
#define MAX(a, b) (((a)>(b)) ? (a) : (b))
#define MIN(a, b) (((a)<(b)) ? (a) : (b))

void download_vsx(){
 FILE *f;
 f=fopen("lib/catalogs/vsx.dat","r");
 if(f==NULL){
  fprintf(stderr,"Downloading VSX catalog!\n");
  system("cd lib/catalogs/ ; wget -c ftp://cdsarc.u-strasbg.fr/pub/cats/B/vsx/vsx.dat.gz ; gunzip vsx.dat.gz");
 }
 else{
  fclose(f);
  return;
 }

 return;
}

void search_vsx(double target_RA_deg, double target_Dec_deg){
 FILE *vsx_dat;
 char name[32];
 char RA_char[32];
 char Dec_char[32];
 double RA_deg,Dec_deg,RA1_rad,RA2_rad,DEC1_rad,DEC2_rad;
 char type[32];
 char descr[128];
 char string[256];
 int i,j;
 
 double distance_deg;
 
 int is_found=0;
 
 download_vsx();
 vsx_dat=fopen("lib/catalogs/vsx.dat","r");
 if( NULL==vsx_dat ){
  fprintf(stderr,"ERROR: Cannot open vsx.dat\n");
  exit(1);
 }
 while(NULL!=fgets(string,256,vsx_dat)){

  for(j=0,i=51;i<60;i++,j++)Dec_char[j]=string[i];
  Dec_char[j]='\0';

  Dec_deg=atof(Dec_char);
  if( fabs(target_Dec_deg-Dec_deg)>VSX_SEARCH_RADIUS_DEG )continue;

  for(j=0,i=8;i<38;i++,j++)name[j]=string[i];
  name[j]='\0';
  
  for(j=0,i=41;i<50;i++,j++)RA_char[j]=string[i];
  RA_char[j]='\0';
  for(j=0,i=61;i<70;i++,j++)type[j]=string[i];
  for(j=0;j<92-61;j++)
   if(type[j]==' ')type[j]='\0';
  for(j=0,i=91;i<strlen(string);i++,j++)descr[j]=string[i];
  descr[j]='\0';
  
  RA_deg=atof(RA_char);
 
  
  RA1_rad=RA_deg*3600/206264.8;
  RA2_rad=target_RA_deg*3600/206264.8;
  DEC1_rad=Dec_deg*3600/206264.8;
  DEC2_rad=target_Dec_deg*3600/206264.8;
       
  distance_deg=acos(cos(DEC1_rad)*cos(DEC2_rad)*cos(MAX(RA1_rad,RA2_rad)-MIN(RA1_rad,RA2_rad))+sin(DEC1_rad)*sin(DEC2_rad))*206264.8/3600.0;
 
  if( distance_deg<VSX_SEARCH_RADIUS_DEG ){
   if( is_found==0 )fprintf(stdout,"The object was <font color=\"green\">found</font> in <font color=\"blue\">VSX</font>\n");
   is_found=1;
   fprintf(stdout,"%2.0lf\"  %s\nType: %s\n#   Max.           Min./Amp.       JD0           Period\n%s",distance_deg*3600.0,name,type,descr);
  }
 }
 if(is_found==0)
  fprintf(stdout,"The object was <font color=\"red\">not found</font> in <font color=\"blue\">VSX</font>\n");
 
 fclose(vsx_dat);
 
 return;
}

int main(int argc, char **argv){

 if( argc<3 ){
  fprintf(stderr,"Usage: %s 12.345 67.890\n",argv[0]);
  return 1;
 }

 double target_RA_deg=atof(argv[1]);
 double target_Dec_deg=atof(argv[2]);

 search_vsx( target_RA_deg, target_Dec_deg);

 return 0;
}
