#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "../fitsio.h"

int main(int argc,char *argv[]){
 /*    */
 fitsfile *fptr;       /* pointer to the FITS file; defined in fitsio.h */
 long  fpixel = 1;
 long naxes[2];
 long testX,testY;
   
 int status = 0, anynul=0,nullval=0;
 unsigned short *image_array;
 unsigned short *dark_array;
 unsigned short *result_image_array;
 
 /* ----- */
 int i;
 int bitpix2;
 /* --      -- */
 char *key[10000];
 int No_of_keys; 
 int keys_left;  
 int ii;

 if( argc<3 ){
  fprintf(stderr,"Not enough arguments... :(\n  Usage: ./ms image.fit dark.fit result.fit\n");  
  exit(1);
 }
  
 /*   */
 fprintf(stderr,"Exploring image header: %s \n",argv[1]);
 ffopen(&fptr, argv[1], 0 , &status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 fits_read_key(fptr,TLONG,"NAXIS1",&naxes[0],NULL,&status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 fits_read_key(fptr,TLONG,"NAXIS2",&naxes[1],NULL,&status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 //       
 fits_get_hdrspace(fptr,&No_of_keys,&keys_left,&status);
 fprintf(stderr,"Header: %d keys total, %d keys left\n",No_of_keys,keys_left);
 for(ii=1;ii<No_of_keys;ii++){
  key[ii]=malloc(80*sizeof(char));
  fits_read_record(fptr,ii,key[ii],&status);
  fprintf(stderr,"Record %d: \"%s\" status=%d\n",ii,key[ii],status);
  fits_report_error(stderr, status);  /* print out any error messages */
  status=0; // continue on any errors at this stage
 }
 fits_get_img_type(fptr, &bitpix2, &status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 
 /*   */
 fprintf(stderr,"Allocating mamory for image, dark and result arrays...\n");
 image_array=malloc(naxes[0] * naxes[1] * sizeof(short));
 dark_array=malloc(naxes[0] * naxes[1] * sizeof(short));
 result_image_array=malloc(naxes[0] * naxes[1] * sizeof(short));
 /*    */
 fits_read_img(fptr, TUSHORT, 1, naxes[0]*naxes[1],&nullval,image_array, &anynul, &status);                 
 fprintf(stderr,"Reading image %s %ld %ld  %d bitpix\n",argv[1],naxes[0],naxes[1],bitpix2);      
 ffclos(fptr,&status);
 fits_report_error(stderr, status);  /* print out any error messages */
 status=0;

 /*   */
 ffopen(&fptr, argv[2], 0 , &status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 fits_read_key(fptr,TLONG,"NAXIS1",&testX,NULL,&status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status);
 fits_read_key(fptr,TLONG,"NAXIS2",&testY,NULL,&status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status != 0 ) exit(status); 
 if( testX != naxes[0] || testY != naxes[1]){
  fprintf(stderr,"Image frame and dark frame must have same dimensions!\n");
  exit(1);
 }
 fits_get_img_type(fptr, &bitpix2, &status);
 fits_read_img(fptr, TUSHORT, 1, naxes[0]*naxes[1],&nullval,dark_array, &anynul, &status);
 fprintf(stderr,"Reading dark frame %s %ld %ld  %d bitpix\n",argv[2],testX,testY,bitpix2);
 ffclos(fptr,&status);
 fits_report_error(stderr, status);  /* print out any error messages */      
 status=0;
 /*    */
 for(i=0;i<naxes[0]*naxes[1];i++){
  /* Try to avoid messing up overscan region */
  if( dark_array[i]<image_array[i] )
   result_image_array[i]=(unsigned short)(image_array[i]-dark_array[i]);
  else
   result_image_array[i]=(unsigned short)(dark_array[i]-image_array[i]);
  if ( result_image_array[i]>65000 )
   result_image_array[i]=65535;
 }
 free(image_array);
 free(dark_array);   
 fits_create_file(&fptr, argv[3], &status);   /* create new file */
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status!=0 ) exit(1);
 fits_create_img(fptr, USHORT_IMG, 2, naxes, &status);
 fits_write_img(fptr, TUSHORT, fpixel, naxes[0]*naxes[1], result_image_array, &status);
 /* --   -- */
 for(ii=1;ii<No_of_keys;ii++){
  fits_write_record(fptr,key[ii],&status);
 }
 fits_delete_key(fptr,"SIMPLE",&status);
 fits_delete_key(fptr,"BITPIX",&status);
 fits_delete_key(fptr,"NAXIS",&status); 
 fits_delete_key(fptr,"NAXIS1",&status);
 fits_delete_key(fptr,"NAXIS2",&status);
 fits_delete_key(fptr,"EXTEND",&status);
 fits_delete_key(fptr,"COMMENT",&status);
 fits_delete_key(fptr,"COMMENT",&status);
 fits_delete_key(fptr,"BZERO",&status);  
 fits_delete_key(fptr,"BSCALE",&status); 
 fits_write_history(fptr, argv[1],  &status);
 fits_write_history(fptr, argv[2],  &status);
 fits_report_error(stderr, status);  /* print out any error messages */
 ffclos (fptr, &status);
 free(result_image_array);
 fprintf(stderr,"Dark frame is subtracted, output is written to %s :)\n\n",argv[3]);
 return 0;
}
