#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_errno.h>

#include "limits.h"

int get_number_of_measured_images_from_vast_summary_log();

int main (int argc, char **argv){
 DIR *dp;
 struct dirent *ep;

 FILE *lightcurvefile;
 double jd,mag,merr,x,y,app;
 char string[FILENAME_LENGTH];
 
 int number_of_measured_images_from_vast_summary_log=get_number_of_measured_images_from_vast_summary_log();
 int n_points_to_drop=MIN((int)(0.1*number_of_measured_images_from_vast_summary_log),STAT_NDROP);
 
 double x_ref,y_ref;
 double m_mean,sigma_series;
 int i,j; // just counters
 double *m=malloc(MAX_NUMBER_OF_OBSERVATIONS*sizeof(double));
 
 dp = opendir ("./");
 if (dp != NULL){
  //fprintf(stderr,"Working...\nPlease, PLEASE, be patient!!!\n");
  while( ep=readdir(dp) ){
   if( strlen(ep->d_name)<12 )continue; // make sure the filename is not too short for the following tests
   if( ep->d_name[0]=='o' && ep->d_name[1]=='u' && ep->d_name[2]=='t' && ep->d_name[strlen(ep->d_name)-1]=='t' && ep->d_name[strlen(ep->d_name)-2]=='a' && ep->d_name[strlen(ep->d_name)-3]=='d' ){
    //puts (ep->d_name);
    lightcurvefile=fopen(ep->d_name,"r");
    if( NULL==lightcurvefile ){
     fprintf(stderr,"ERROR: Can't open file %s\n",ep->d_name);
     exit(1);
    }
    i=0;
    while(-1<fscanf(lightcurvefile,"%lf %lf %lf %lf %lf %lf %s",&jd,&mag,&merr,&x,&y,&app,string)){
     if(i==0){x_ref=x;y_ref=y;}
     if( 1==isnormal(mag) ){
      if( mag!=0.0 ){
       m[i]=mag;
       i++;
       if(i>=MAX_NUMBER_OF_OBSERVATIONS){fprintf(stderr,"ERROR (src/create_data.c): i>=MAX_NUMBER_OF_OBSERVATIONS=%d\n",MAX_NUMBER_OF_OBSERVATIONS);fclose(lightcurvefile);exit(1);}
      }
     }
    }
    fclose(lightcurvefile);
    if( i<MIN(TESTOR_MIN_NUMBER_OF_POINTS,(int)(0.5*number_of_measured_images_from_vast_summary_log)) )continue;
    gsl_sort(m,1,i);
    /* Drop points */
    if( i>STAT_MIN_NUMBER_OF_POINTS_FOR_NDROP && i>2*n_points_to_drop ){
     i-=n_points_to_drop;   
     for(j=0;j<i;j++){
      m[j]=m[j+n_points_to_drop];
     }
     i-=n_points_to_drop;
    }    
    m_mean=gsl_stats_median_from_sorted_data(m,1,i);
    sigma_series=gsl_stats_sd(m,1,i);
    if( sigma_series==0.0 || 0==isnormal(sigma_series) )continue;
    
    fprintf(stdout,"%8.4lf %6.4lf %8.3lf %8.3lf %s\n",m_mean,sigma_series,x_ref,y_ref,ep->d_name);
    
   }
  }
  (void) closedir (dp);
 }
 else
  perror("Couldn't open the directory");
  
  free(m);
  
  fprintf(stderr,"All done!  =)\n");                                         

  return 0;
}
