#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_errno.h>

#include "cpgplot.h"

#include "setenv_local_pgplot.h"

#include "limits.h"

#include "photocurve.h"

int wpolyfit(double *datax, double *datay, double *dataerr, int n, double *poly_coeff);

int main(int argc, char **argv){
 char calibfilename[FILENAME_LENGTH];
 FILE *calibfile;
 double *insmag;
 float *finsmag;
 double *insmagerr;
 float *finsmagerr;
 double *catmag;
 float *fcatmag;
 double *w;
 double cov00,cov01,cov11,cov02,cov12,cov22,sumsqres;
 int n_stars=0;
 int j,k;
 
 double poly_coeff[8];
 double A=0;
 double B=0;
 double C=0;
 
 double sum1,sum2;
 
 float mincatmag,maxcatmag,mininstmag,maxinstmag;

 float *computed_x;
 float *computed_y;
 
 /* PGPLOT vars */
 float curX,curY; 
 float curX2,curY2;
 char curC;
 char PGPLOT_CONTROL[100];
 int change_limits_trigger=0;
 float new_X1,new_X2,old_X1,old_X2;
 float new_Y1,new_Y2,old_Y1,old_Y2;
 char header_str[512];
 char header_str2[512];
 /* ----------- */

 /* switch */
 int weights_on=1;
 int fit_function=2; // 1 - line, 2 - parabola, 3 - line with a=1, 4 - "photocurve", 5 - "inverse photocurve" (will be set automatically if fits better than "photocurve")

 /* remove points */
 int remove_best_j;
 float best_dist=99999;
 float y_to_x_scaling_factor; // to compensate for the difference in scale between the two axes
 
 /* Allocate memory */ 
 insmag=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(double));
 finsmag=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(float));
 insmagerr=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(double));
 finsmagerr=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(float));
 catmag=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(double));
 fcatmag=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(float));
 w=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(double)); 
 computed_x=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(float));
 computed_y=malloc(MAX_NUMBER_OF_STARS_MAG_CALIBR*sizeof(float));

 if( argc==1 )
  strncpy(calibfilename,"calib.txt",FILENAME_LENGTH);
 else
  strncpy(calibfilename,argv[1],FILENAME_LENGTH);

 /* Read data file */
 calibfile=fopen(calibfilename,"r");
 if( NULL==calibfile ){
  fprintf(stderr,"ERROR: Can't open file %s\n",calibfilename);
  exit(1);
 }
 while(-1<fscanf(calibfile,"%lf %lf %lf",&insmag[n_stars], &catmag[n_stars], &insmagerr[n_stars])){
  finsmag[n_stars]=(float)insmag[n_stars];
  fcatmag[n_stars]=(float)catmag[n_stars];
  finsmagerr[n_stars]=(float)insmagerr[n_stars];
  w[n_stars]=1.0/(insmagerr[n_stars]*insmagerr[n_stars]);
  n_stars++;
 }
 fclose(calibfile);

 /* Set limits for plotting */
 mininstmag=maxinstmag=finsmag[0];
 mincatmag=maxcatmag=fcatmag[0];
 for(j=0;j<n_stars;j++){
  mininstmag=MIN(mininstmag,finsmag[j]);
  maxinstmag=MAX(maxinstmag,finsmag[j]);
  mincatmag=MIN(mincatmag,fcatmag[j]);
  maxcatmag=MAX(maxcatmag,fcatmag[j]);
 }
// fprintf(stderr,"mininstmag=%lf maxinstmag=%lf mincatmag=%lf maxcatmag=%lf",mininstmag,maxinstmag,mincatmag,maxcatmag);

 /* if n_stars<5 set fit_function=3 (line with fixed a) */
 if( n_stars<5 )
  fit_function=3;


 /* if n_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR set fit_function=1 (line) */
 if( n_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR )
  fit_function=1;

 /* Special one-star mode */
 if( n_stars==1 ){
  fprintf(stdout,"%lf %lf %lf\n",0.0,1.0,(catmag[0])-(insmag[0]));
  return 0;
 }


 /* GUI */
 setenv_localpgplot(argv[0]);
 strcpy(PGPLOT_CONTROL,"/XW");
 if(cpgbeg(0, PGPLOT_CONTROL, 1, 1) != 1)
  return EXIT_FAILURE;

 do{

 cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
  
 if( change_limits_trigger==0 ){
   cpgpage();
 }

 cpgeras();
 cpgsvp(0.08, 0.95, 0.1, 0.9);
 if( change_limits_trigger==2 || change_limits_trigger==0 ){
  old_Y1=mincatmag-(maxcatmag-mincatmag)/10;                            
  old_Y2=maxcatmag+(maxcatmag-mincatmag)/10;
  new_Y1=old_Y1;                
  new_Y2=old_Y2;

  old_X1=mininstmag-(maxinstmag-mininstmag)/10;
  old_X2=maxinstmag+(maxinstmag-mininstmag)/10;
  new_X1=old_X1;                   
  new_X2=old_X2;
 }

 cpgswin(new_X1, new_X2, new_Y1, new_Y2);
 cpgscr( 0, 0.08, 0.08, 0.09);  /* set background */
 cpgsci(0);
 cpgrect(new_X1, new_X2, new_Y1, new_Y2);
 cpgsci(1);
 cpgscf(1);
 cpgbox("BCNST1", 0.0 , 0, "BCNST1", 0.0, 0);

 /* Processing */
 A=B=C=0.0;
 cov22=cov11=cov00=0.0;
 
 if( argc==3 ){
  /* Just read A, B and C from input file */
  strncpy(calibfilename,argv[2],FILENAME_LENGTH);
  fprintf(stderr,"Reading fitting coefficients from file %s\n",calibfilename);
  calibfile=fopen(calibfilename,"r");
  if( NULL==calibfile ){
   fprintf(stderr,"ERROR: Can't open file %s\n",calibfilename);
   exit(1);
  }
  //fscanf(calibfile,"%lf %lf %lf",&A,&B,&C);
  fscanf(calibfile,"%lf %lf %lf %lf %lf",&poly_coeff[4],&poly_coeff[3],&poly_coeff[2],&poly_coeff[1],&poly_coeff[0]);
  if( poly_coeff[4]==4.0 || poly_coeff[4]==5.0 ){
   fit_function=(int)poly_coeff[4];
  }
  else{
   A=poly_coeff[2];
   B=poly_coeff[1];
   C=poly_coeff[0];
   if( A==0.0 )
    fit_function=1;
   else
    fit_function=2;
   fprintf(stderr,"%lf %lf %lf   %d\n",A,B,C,fit_function);
  }
  fclose(calibfile);
 }
 else{
  /* Compute A, B and C using current fitting function */
  if( fit_function==1 ){
   gsl_fit_wlinear(insmag,1,w,1,catmag,1,n_stars,&C,&B,&cov00,&cov01,&cov11,&sumsqres);
  }
  if( fit_function==2 ){
   wpolyfit(insmag, catmag, insmagerr, n_stars, poly_coeff);
   A=poly_coeff[2];
   B=poly_coeff[1];
   C=poly_coeff[0];
   cov00=poly_coeff[5];
   cov11=poly_coeff[6];
   cov22=poly_coeff[7];
  }
  if( fit_function==3 ){
   B=1.0;
   sum1=sum2=0.0;
   for(j=0;j<n_stars;j++){
    sum1+=w[j]*(catmag[j]-insmag[j]);
    sum2+=w[j];
   }
   C=sum1/sum2;
  }
  if( fit_function==4 || fit_function==5 ){
   fit_photocurve(insmag, catmag, insmagerr, n_stars, poly_coeff, &fit_function);
  }
 }
 
 /* Print plot header */                  
 cpgscf(1);
 cpgsch(1.0); /* make lables with normal characters */  
 sprintf(header_str,"fitting function: y=");
 if( A!=0.0 ){
  sprintf(header_str2,"%lf\\(0729)x\\u2\\d",A);
  strcat(header_str,header_str2);
  sprintf(header_str2,"%+lf\\(0729)x",B);
  strcat(header_str,header_str2);
 }
 else{
  sprintf(header_str2,"%lf\\(0729)x",B);
  strcat(header_str,header_str2);
 }
 sprintf(header_str2,"%+lf",C);
 strcat(header_str,header_str2);

 if( fit_function==4 ){
  sprintf(header_str,"fitting function: y=%.5lf*log\\d10\\u(10\\u%.5lf*(x-(%.5lf))\\d+1)+(%.5lf)",poly_coeff[0],poly_coeff[1],poly_coeff[2],poly_coeff[3]); 
 }
 if( fit_function==5 ){
  sprintf(header_str,"fitting function: x=%.5lf*log\\d10\\u(10\\u%.5lf*(y-(%.5lf))\\d+1)+(%.5lf)",poly_coeff[0],poly_coeff[1],poly_coeff[2],poly_coeff[3]); 
 }


 /* If we don't read parameters from input file... */
 if( argc!=3 && fit_function!=4 && fit_function!=5 ){
  sprintf(header_str2,"  press 'P' to change function");
  strcat(header_str,header_str2);
 }
 
 
 //sprintf(header_str,"fitting function: %lf * x\\u2\\d + %lf * x + %lf", A, B, C);
 cpgmtxt( "T", 2.0, 0.0, 0.0, header_str);
 sprintf(header_str,"use weights: ");
 if( weights_on==1 )
  strcat(header_str,"yes");
 else
  strcat(header_str," no");
 /* If we don't read parameters from input file... */
 if( argc!=3 ){
  strcat(header_str,"  press 'W' to change it");
 }
 cpgmtxt( "T", 1.0, 0.0, 0.0, header_str);
 cpglab("Instrumental magnitude","Catalog magnitude", "");
 cpgscr( 1, 1.0, 1.0, 1.0);
 cpgsch(1.0);

 /* Compute best fit function */
 if( fit_function!=4 && fit_function!=5 ){
  /* line or parabola */
  computed_x[0]=mininstmag;
  computed_y[0]=A*computed_x[0]*computed_x[0]+B*computed_x[0]+C;
  for(j=1;j<MAX_NUMBER_OF_STARS_MAG_CALIBR;j++){
   computed_x[j]=computed_x[j-1]+(maxinstmag-mininstmag)/MAX_NUMBER_OF_STARS_MAG_CALIBR;
   computed_y[j]=A*computed_x[j]*computed_x[j]+B*computed_x[j]+C;
  }
 }
 else{
  /* photocurve or inverse photocurve */
  computed_x[0]=mininstmag;
  computed_y[0]=(float)eval_photocurve( (double)computed_x[0], poly_coeff, fit_function);
  for(j=1;j<MAX_NUMBER_OF_STARS_MAG_CALIBR;j++){
   computed_x[j]=computed_x[j-1]+(maxinstmag-mininstmag)/MAX_NUMBER_OF_STARS_MAG_CALIBR;
   computed_y[j]=(float)eval_photocurve( (double)computed_x[j], poly_coeff, fit_function);
  }
 }

 /* Draw data points */
 cpgsci(2); // red
 for(j=0;j<n_stars;j++){
  cpgerr1(6, finsmag[j], fcatmag[j], finsmagerr[j], 1.0);
 }
 cpgpt(n_stars, finsmag, fcatmag, 17);
 
 /* Draw best fit curve */
 cpgsci(3); // green
 cpgline(MAX_NUMBER_OF_STARS_MAG_CALIBR, computed_x, computed_y);
  
 cpgsci(5);
 cpgcurs(&curX, &curY, &curC);
 /* Zoom */
 if( curC=='Z' || curC=='z' ){
  cpgband( 2, 0, curX, curY, &curX2, &curY2, &curC);
  if( new_X1!=0.0 ){
   old_X1=new_X1;   
   old_X2=new_X2;   
  }
  new_X1=curX;
  new_Y1=curY;
                              
  if( curC=='Z' || curC=='z' )
   curC='D';

  if( curX2>new_X1 )
   new_X2=curX2;
  else{
   new_X2=new_X1;
   new_X1=curX2; 
  }
  if( curY2>new_Y1 )
   new_Y2=curY2;
  else{
   new_Y2=new_Y1;
   new_Y1=curY2; 
  }

 } /* End of Zoom */

 change_limits_trigger=1;
 
 if( curC=='D' || curC=='d' ){
  change_limits_trigger=2;
 }
 
 /* Use/don't use weights */
 if( curC=='W' || curC=='w' ){
  if( weights_on==1 ){
   weights_on=0;
   for(j=0;j<n_stars;j++){
    w[j]=1.0;
    insmagerr[j]=0.01;
   }
  }
  else{
   weights_on=1;
   for(j=0;j<n_stars;j++){
    insmagerr[j]=(double)finsmagerr[j];
    w[j]=1.0/(insmagerr[j]*insmagerr[j]);
   }
  }
 }
 
 /* Change fitting function */
 if( curC=='P' || curC=='p' ){
  fit_function++;
  if( fit_function>4 )
   fit_function=1;
 }


 y_to_x_scaling_factor=fabsf(new_X2-new_X1)/fabsf(new_Y2-new_Y1);

 /* Remove point */ 
 if( curC=='R' || curC=='r' ){ 
  /* find closest point */
  best_dist=999.9*999.9;
  for(j=0;j<n_stars;j++){
   if( (finsmag[j]-curX)*(finsmag[j]-curX)+(fcatmag[j]-curY)*(fcatmag[j]-curY)*y_to_x_scaling_factor*y_to_x_scaling_factor<best_dist ){
   //if( (finsmag[j]-curX)*(finsmag[j]-curX)+(fcatmag[j]-curY)*(fcatmag[j]-curY)<best_dist ){
   // best_dist=(finsmag[j]-curX)*(finsmag[j]-curX)+(fcatmag[j]-curY)*(fcatmag[j]-curY);
    best_dist=(finsmag[j]-curX)*(finsmag[j]-curX)+(fcatmag[j]-curY)*(fcatmag[j]-curY)*y_to_x_scaling_factor*y_to_x_scaling_factor;
    remove_best_j=j;
   }
  }
  /* remove it */
  //for(j=remove_best_j;j<n_stars-1;j++){
  for(j=remove_best_j;j<n_stars;j++){
   insmag[j]=insmag[j+1];
   finsmag[j]=finsmag[j+1];
   catmag[j]=catmag[j+1];
   fcatmag[j]=fcatmag[j+1];
   insmagerr[j]=insmagerr[j+1];
   finsmagerr[j]=finsmagerr[j+1];
   w[j]=w[j+1];
  }
  n_stars--; // ?  
 }

 /* Remove all data points within a user-specified rectangular region */
 if( curC=='C' || curC=='c' ){
  cpgsci(2);
  cpgband( 2, 0, curX, curY, &curX2, &curY2, &curC);
  // last chance to cancel!
  if( curC!='X' && curC!='x' ){
   for(j=0;j<n_stars;j++){
    if( finsmag[j]>MIN(curX,curX2) && finsmag[j]<MAX(curX,curX2) && fcatmag[j]>MIN(curY,curY2) && fcatmag[j]<MAX(curY,curY2) ){
     /* remove it */
     for(k=j;k<n_stars;k++){
      insmag[k]=insmag[k+1];
      finsmag[k]=finsmag[k+1];
      catmag[k]=catmag[k+1];
      fcatmag[k]=fcatmag[k+1];
      insmagerr[k]=insmagerr[k+1];
      finsmagerr[k]=finsmagerr[k+1];
      w[k]=w[k+1];
     }
     n_stars--; // ?
     j--;
    }
   }
  }
  curC=' ';
 }


 
 }while( curC!='X' );

 if( fit_function!=4 && fit_function!=5 ) 
  fprintf(stdout,"%lf %lf %lf\n", A, B, C);
 else
  fprintf(stdout,"%d %lf %lf %lf %lf\n", fit_function, poly_coeff[0], poly_coeff[1], poly_coeff[2], poly_coeff[3]);
 
 return 0;
}
