/*
 The function described in this file will try to determine the number of CPU cores 
 on the local machine by parsing /proc/cpuinfo file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "limits.h" // for N_FORK which is defined in limits.h

int get_number_of_cpu_cores(){
 char string[1024]; // string in the /proc/cpuinfo file
 int n_cores=N_FORK; // default number of cores
 int i; // a counter 
 FILE *proc_cpuinfo_file;
 proc_cpuinfo_file=fopen("/proc/cpuinfo","r");
 if( NULL==proc_cpuinfo_file ){
  fprintf(stderr,"WARNING: cannot open /proc/cpuinfo !\nThe number of CPU cores is set to the default value of %d.\nThis default value is defined in src/limits.h\n",n_cores);
  return n_cores;
 }
 n_cores=0;
 // for each string in /proc/cpuinfo ...
 while(NULL!=fgets(string,1024,proc_cpuinfo_file)){
  // cut the first work out of the string
  for(i=0;i<strlen(string);i++)
   if( string[i]==':' ){
    string[i]='\0';
    break;
   }
  // test if this the right key
  if( 0==strcmp(string,"processor\t") )n_cores++;
 }
 fclose(proc_cpuinfo_file);
 if( n_cores==0 ){
  n_cores=N_FORK;
  fprintf(stderr,"WARNING: cannot parse /proc/cpuinfo !\nThe number of CPU cores is set to the default value of %d.\nThis default value is defined in src/limits.h\n",n_cores);
  return n_cores; 
 }
 fprintf(stderr,"The number of processor cores determined from /proc/cpuinfo is %d.\n",n_cores);
 return n_cores;
}
