#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "fitsio.h"

#include "limits.h"

double convert_jdUT_to_jdTT(double jdUT, int *timesys){
 FILE *tai_utc_dat;
 double jdTT;
 double *jd_leap_second;
 jd_leap_second=malloc(MAX_NUMBER_OF_LEAP_SECONDS*sizeof(double));
 if( jd_leap_second==NULL ){fprintf(stderr,"ERROR: in convert_jdUT_to_jdTT() can't allocate memory for jd_leap_second\n");exit(1);}
 double *TAI_minus_UTC;
 TAI_minus_UTC=malloc(MAX_NUMBER_OF_LEAP_SECONDS*sizeof(double));
 if( TAI_minus_UTC==NULL ){fprintf(stderr,"ERROR: in convert_jdUT_to_jdTT() can't allocate memory for TAI_minus_UTC\n");exit(1);}
 double tai_utc;
 char str1[256], str2[256];
 int i;
 int n_leap_sec=0;
 
 double MJD=jdUT-2400000.5; // for leap second calculation before 1972 JAN  1
 double *MJD0;
 MJD0=malloc(MAX_NUMBER_OF_LEAP_SECONDS*sizeof(double));
 if( MJD0==NULL ){fprintf(stderr,"ERROR: in convert_jdUT_to_jdTT() can't allocate memory for MJD0\n");exit(1);}
 double *leap_second_rate;
 leap_second_rate=malloc(MAX_NUMBER_OF_LEAP_SECONDS*sizeof(double));
 if( leap_second_rate==NULL ){fprintf(stderr,"ERROR: in convert_jdUT_to_jdTT() can't allocate memory for leap_second_rate\n");exit(1);}
 
 /* 
   Read the file with leap seconds lib/tai-utc.dat
   up-to-date version of this file is available at 
   http://maia.usno.navy.mil/ser7/tai-utc.dat
 */
 tai_utc_dat=fopen("lib/tai-utc.dat","r");
 if( NULL==tai_utc_dat ){
  fprintf(stderr,"ERROR: can't open file lib/tai-utc.dat\n");
  exit(1);
 }
 while( NULL!=fgets(str1, 256, tai_utc_dat) ){
  for(i=17;i<26;i++)str2[i-17]=str1[i];
  str2[i-17]='\0';
  jd_leap_second[n_leap_sec]=atof(str2);
  for(i=37;i<48;i++)str2[i-37]=str1[i];
  str2[i-37]='\0';
  TAI_minus_UTC[n_leap_sec]=atof(str2);
  for(i=60;i<66;i++)str2[i-60]=str1[i];
  str2[i-60]='\0';
  MJD0[n_leap_sec]=atof(str2);
  for(i=70;i<79;i++)str2[i-70]=str1[i];
  str2[i-70]='\0';
  leap_second_rate[n_leap_sec]=atof(str2);
  n_leap_sec++;
 }
 fclose(tai_utc_dat);

 tai_utc=TAI_minus_UTC[0];
 for(i=1;i<n_leap_sec;i++)
  if( jdUT>=jd_leap_second[i] )tai_utc= TAI_minus_UTC[i] + (MJD - MJD0[i]) * leap_second_rate[i];  //tai_utc=TAI_minus_UTC[i];
  
 jdTT=jdUT+(32.184+tai_utc)/86400; // TT = TAI + 32.184
                                   // TAI = UTC + leap_seconds

 /* Set marker that time system was changed */ 
 (*timesys)=2; // TT

 free(jd_leap_second);
 free(TAI_minus_UTC);
 free(MJD0);
 free(leap_second_rate);

 return jdTT;
}

int fitsfile_read_check(char *fitsfilename){
	int status = 0; //for cfitsio routines
	fitsfile *fptr; /* pointer to the FITS file; defined in fitsio.h */
	fits_open_file(&fptr, fitsfilename, READONLY, &status);
	fits_report_error(stderr, status);  /* print out any error messages */
	if (0 != status)
		return status;
	fits_close_file(fptr, &status);
	return 0;
}

// This function will handle '25/12/2011' style DATE-OBS
void fix_DATEOBS_STRING(char *DATEOBS){
 int i,j,date_part; // counters
 char substring_day[32];
 char substring_month[32];
 char substring_year[32];
 int day,month,year;
 
 // check if this is an empty string (and assume that the date will be provided as 'JD' keyword)
 if(0==strlen(DATEOBS))return;
 
 // check if this is a normal '2004-07-05' style DATE-OBS
 for(i=0;i<strlen(DATEOBS);i++)
  // if yes - do notheing
  if( DATEOBS[i]=='-' )return;
 
 // Parse '25/12/2011' style DATE-OBS
 for(i=0,j=0,date_part=1;i<strlen(DATEOBS);i++){
  if( DATEOBS[i]=='/' ){
   if(date_part==1)substring_day[j]='\0';
   if(date_part==2)substring_month[j]='\0';
   if(date_part==3)substring_year[j]='\0';
   date_part++;
   j=0;
   continue;
  }
  if(date_part==1){
   substring_day[j]=DATEOBS[i];j++;
  }
  if(date_part==2){
   substring_month[j]=DATEOBS[i];j++;
  }
  if(date_part==3){
   substring_year[j]=DATEOBS[i];j++;
  }
 }
 substring_year[4]='\0';
 
 if( date_part==1 ){
  fprintf(stderr,"ERROR: cannot parse DATE-OBS keyword!\n");
  exit(1);
 }
 
 // Print result
 //fprintf(stderr,"_%s_ _%s_ _%s_   _%s_\n",substring_day,substring_month,substring_year,DATEOBS);
 day=atoi(substring_day);
 month=atoi(substring_month);
 year=atoi(substring_year);
 sprintf(DATEOBS,"%d-%02d-%02d",year,month,day);

 return;
}

int  gettime(char *fitsfilename, double *JD, int *timesys, int convert_timesys_to_TT, double *dimX, double *dimY, char *stderr_output, char *log_output, int param_nojdkeyword, int verbose) {
        
	/* Variables for time */
	int status = 0; //for cfitsio routines
	int j;
	int jj;
	time_t Vrema;
	struct tm structureTIME;
	//struct tm *ukaz_struTIME;
	char Vr_h[10], Vr_m[10], Vr_s[10];
	char Da_y[10], Da_m[10], Da_d[10];

	char DATEOBS[32], TIMEOBS[32], TIMESYS[32];
	char DATEOBS_COMMENT[2048]; // make it long, just in case
	char tymesys_str_in[32];
	char tymesys_str_out[32];
	double inJD = 0;

	/* fitsio */
	fitsfile *fptr; /* pointer to the FITS file; defined in fitsio.h */
	//long  fpixel = 1, naxis = 2, nelements;//, exposure;
	double exposure;
	/*End of time variables */
	long naxes[2];

        // LOG-files
	//FILE *vast_image_details;
        
        //
        
        int date_parsed=0; // 0 - date is not parsed, 1 - date is parsed successfully

	if(verbose==1)fprintf(stderr,"Processing  %s\n",fitsfilename);
	
	/* Extract data from fits header */
	fits_open_file(&fptr, fitsfilename, READONLY, &status);
	if ( 0!=status ){
         fits_report_error(stderr, status);  /* print out any error messages */
         return status;
        }
        
        // Get image dimentions
	fits_read_key(fptr, TLONG, "NAXIS1", &naxes[0], NULL, &status);
	if ( 0!= status ){
	 fits_report_error(stderr, status); 
	 fits_close_file(fptr, &status); // close file
	 fprintf(stderr,"ERROR: gettime() - can't get image dimensions from NAXIS1 keyword!\n");
	 return status;
        }
	fits_read_key(fptr, TLONG, "NAXIS2", &naxes[1], NULL, &status);
	if ( 0!=status ){
	 fits_report_error(stderr, status);
	 fits_close_file(fptr, &status); // close file
	 fprintf(stderr,"ERROR: gettime() - can't get image dimensions from NAXIS2 keyword!\n");
	 return status;
        }
        
	fits_read_key(fptr, TSTRING, "DATE-OBS", DATEOBS, DATEOBS_COMMENT, &status);
	if(status==0)date_parsed=1;
	
	// if it does not exist at all, try DATE
	if (status == 202){
	 date_parsed=0;
	 if(verbose==1)fprintf(stderr,"WARNING: DATE-OBS keyword not found, trying DATE...\n");
	 status=0;
	 fits_read_key(fptr, TSTRING, "DATE", DATEOBS, DATEOBS_COMMENT, &status);
	 if(status==0)date_parsed=1;
        }
	if( strlen(DATEOBS)<11 && strlen(DATEOBS)>1 && date_parsed==1 ){
	        fix_DATEOBS_STRING(DATEOBS); // handle '25/12/2011' style DATE-OBS
		fits_read_key(fptr, TSTRING, "TIME-OBS", TIMEOBS, NULL, &status);
		if (0 == strcmp(TIMEOBS, "")) {
                        status = 0; // guess need to set it here or the folowing  FITSIO request will not pass
			if(verbose==1)fprintf(stderr, "Looking for time of observation in START keyword (its format is assumed to be similar to TIME-OBS keyword)\n");
			fits_read_key(fptr, TSTRING, "START   ", TIMEOBS, NULL, &status);
			status = 0; // need to set it here or the folowing  FITSIO request will not pass
			if (0 == strcmp(TIMEOBS, "")) {
			 if(verbose==1)fprintf(stderr, "Looking for time of observation in UT-START keyword (its format is assumed to be similar to TIME-OBS keyword)\n");
                         fits_read_key(fptr, TSTRING, "UT-START", TIMEOBS, NULL, &status);
                         if (0 == strcmp(TIMEOBS, ""))fprintf(stderr,"WARNING! Cannot get proper observing time from FITS header! Assuming it will be provided as JD keyword...\n");
			}
		}
		status = 0;
	} // if( strlen(DATEOBS)<11 && strlen(DATEOBS)>1 && date_parsed==1 ){
	fits_read_key(fptr, TDOUBLE, "EXPTIME", &exposure, NULL, &status);
	if (status == 202) {
		status = 0;
		if(verbose==1)fprintf(stderr, "Looking for exposure in EXPOSURE \n");
		fits_read_key(fptr, TDOUBLE, "EXPOSURE", &exposure, NULL, &status);
		if (status == 202) {
			if(verbose==1)fprintf(stderr, "I can't find a keyword with exposure time! ;(     assuming EXPTIME=0\n");
			exposure = 0.0;
			status = 0;
		}
	}
	fits_report_error(stderr, status);  /* print out any error messages */
	
	/* Look for JD keyword (for Moscow photographic plate scans) */
	status = 0;
	// param_nojdkeyword==1 tells us that JD keyword should be ignored
	// date_parsed==0 means DATE-OBS was not found and parsed
	if ( param_nojdkeyword==0 && date_parsed==0 ){
	 exposure = 0.0; // Assume JD keyword corresponds to the middle of exposure!!!
	 fits_read_key(fptr, TDOUBLE, "JD", &inJD, NULL, &status);
	 if( status!=202 && verbose==1 ){
	  fprintf(stderr,"Getting JD of the middle of exposure from JD keyword: %.5lf\n",inJD);
	  // Check that JD is within the reasonable range
	  if( inJD<EXPECTED_MIN_JD || inJD>EXPECTED_MAX_JD ){
	   fprintf(stderr,"ERROR: JD out of expected range (%.1lf, %.1lf)!\nYou may change EXPECTED_MIN_JD and EXPECTED_MAX_JD in src/limits.h and recompile VaST if you are _really sure_ you know what you are doing...\n", EXPECTED_MIN_JD, EXPECTED_MAX_JD);
	   exit(1);
	  }
	 }
        }
        else
         status=202;
        

	(*dimX) = naxes[0];
	(*dimY) = naxes[1];
	if (status == 202) {
            /* If no JD keyword was found... */

	    /* 
	       Try to guess time system (UTC or TT).
	       Look for TIMESYS keyword, if it is not found 
	       - try to parse DATE-OBS comment.
	     */
	     status = 0;
	     fits_read_key(fptr, TSTRING, "TIMESYS", TIMESYS, NULL, &status);
	     if( status!=202){
	      // TIMESYS keyword found
	      if(verbose==1)fprintf(stderr,"TIMESYS keyword found: %s\n",TIMESYS);
	      if( TIMESYS[0]=='T' && TIMESYS[1]=='T' )
	       (*timesys)=2; // TT
	      else
	       if( TIMESYS[0]=='U' && TIMESYS[1]=='T' )
	        (*timesys)=1; // UT
	       else
	        (*timesys)=0; // UNKNOWN
             }
	     else{
	      // TIMESYS keyword not found, try to parse DATE-OBS comment string
	      
	      // Make sure the string is not empty
              if( strlen(DATEOBS_COMMENT)>1 ){
               // don't start from 0 - if there is no comment, the string will contain 0 characters before \0 !
	       for(j=1;j<strlen(DATEOBS_COMMENT)-1;j++){
	        if( DATEOBS_COMMENT[j]=='U' &&  DATEOBS_COMMENT[j+1]=='T' ){
	         (*timesys)=1; // UT
	         if(verbose==1)fprintf(stderr,"Time system set from the comment to DATE-OBS keyword: %s\n",DATEOBS_COMMENT);
	         break;
                }
	       }
              } // if( strlen(DATEOBS_COMMENT)>1 ){
	       
	      if( (*timesys)!=1 )(*timesys)=0; // UNKNOWN
	     }
	     
	     /* Choose string to describe time system */
	     if( (*timesys)==2 )
              sprintf(tymesys_str_in,"TT");
             else
              if( (*timesys)==1 )
               sprintf(tymesys_str_in,"UT");
              else
               sprintf(tymesys_str_in,"");


                if(verbose==1)fprintf(stderr,"Setting observation date using DATE-OBS keyword: %s\n",DATEOBS);

		for (j = 0;j < 32;j++) {
			if (DATEOBS[j] == 45) {
				Da_y[j] = '\0';
				break;
			}
			Da_y[j] = DATEOBS[j];
		}
		for (j += 1;j < 32;j++) {
			if (DATEOBS[j] == 45) {
				Da_m[j-5] = '\0';
				break;
			}
			Da_m[j-5] = DATEOBS[j];
		}
		for (j += 1;j < 32;j++) {
			if (DATEOBS[j] == '\0' || DATEOBS[j] == 'T') {
				Da_d[j-7] = '\0';
				break;
			}
			Da_d[j-8] = DATEOBS[j];
		}
		//Если время прописано в DATE-OBS после T
		if (DATEOBS[j] == 'T') {
                        if(verbose==1)fprintf(stderr,"Setting observation time using DATE-OBS keyword: %s\n",DATEOBS);
			jj = 0;
			for (j += 1;j < 32;j++) {
				if (DATEOBS[j] == '\0') {
					TIMEOBS[jj] = '\0';
					break;
				}
				TIMEOBS[jj] = DATEOBS[j];
				jj++;
			}

		}
		else
		 if(verbose==1)fprintf(stderr,"Setting observation time using TIME-OBS keyword: %s\n",TIMEOBS);

		for (j = 0;j < 32;j++) {
			if (TIMEOBS[j] == ':') {
				Vr_h[j] = '\0';
				break;
			}
			Vr_h[j] = TIMEOBS[j];
		}
		for (j += 1;j < 32;j++) {
			if (TIMEOBS[j] == ':') {
				Vr_m[j-3] = '\0';
				break;
			}
			Vr_m[j-3] = TIMEOBS[j];
		}
		for (j += 1;j < 32;j++) {
			if (TIMEOBS[j] == '\0' || TIMEOBS[j] == '.') {
				Vr_s[j-6] = '\0';
				break;
			}
			Vr_s[j-6] = TIMEOBS[j];
		}
		Vr_s[2]='\0';
		Vr_m[2]='\0';
		Vr_m[2]='\0';
		Da_d[2]='\0';
		Da_m[2]='\0';
		Da_y[4]='\0';
		structureTIME.tm_sec = atoi(Vr_s);
		structureTIME.tm_min = atoi(Vr_m);
		structureTIME.tm_hour = atoi(Vr_h);
		structureTIME.tm_mday = atoi(Da_d);
		structureTIME.tm_mon = atoi(Da_m) - 1;
		structureTIME.tm_year = atoi(Da_y) - 1900;
		Vrema = timegm(&structureTIME);
		Vrema = (time_t)((double)Vrema + exposure/2.0);
		(*JD) = (double)Vrema/3600.0/24.0+2440587.5;
		
		/* Convert JD(UT) to JD(TT) if needed */
		if( (*timesys)==1 && convert_timesys_to_TT==1 )
		 (*JD) = convert_jdUT_to_jdTT((*JD), timesys);
		
		/* Choose string to describe new time system */
		if( (*timesys)==2 )
	         sprintf(tymesys_str_out,"(TT)");
	        else
                 if( (*timesys)==1 )
                  sprintf(tymesys_str_out,"(UT)");
                 else
                  sprintf(tymesys_str_out,"");
                  
		sprintf(log_output,"exp_start= %02d.%02d.%4d %02d:%02d:%02d  exp= %4.0lf  ", structureTIME.tm_mday, structureTIME.tm_mon+1, structureTIME.tm_year-100+2000, structureTIME.tm_hour, structureTIME.tm_min, structureTIME.tm_sec, exposure);

                if( exposure!=0 )sprintf(stderr_output, "Exposure %3.0lf sec, %02d.%02d.%4d %02d:%02d:%02d %s = JD%s %.5lf mid. exp.\n", exposure, structureTIME.tm_mday, structureTIME.tm_mon+1, structureTIME.tm_year-100+2000, structureTIME.tm_hour, structureTIME.tm_min, structureTIME.tm_sec, tymesys_str_in, tymesys_str_out, (*JD));
                if( exposure==0 )sprintf(stderr_output, "Exposure %3.0lf sec, %02d.%02d.%4d %02d:%02d:%02d %s = JD%s %.5lf\n", exposure, structureTIME.tm_mday, structureTIME.tm_mon+1, structureTIME.tm_year-100+2000, structureTIME.tm_hour, structureTIME.tm_min, structureTIME.tm_sec, tymesys_str_in, tymesys_str_out, (*JD));
	} else {
	        /* Setting pre-calculated JD(UT) mid. exp. from the JD keyword */
	        (*timesys)=1;
	        sprintf(tymesys_str_out,"(UT)");
		(*JD) = inJD;
		exposure = 0.0;
		sprintf(stderr_output, "JD (mid. exp.) %.5lf\n",(*JD));
		sprintf(log_output,"exp_start= %02d.%02d.%4d %02d:%02d:%02d  exp= %4.0lf  ", 0, 0, 0, 0, 0, 0, exposure);

	}
	fits_close_file(fptr, &status); // close file
	
	return 0;
}
