/*
   This program should identify stars with large sigma.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_errno.h>

#include "limits.h"
#include "vast_math.h"

int main(int argc, char **argv){
 FILE *sigma_selection_curve_log;
 FILE *dmsf;
 double m,sigma,X,Y,sum,mmax,mean;
 double *data;
 double *x;
 double *y;
 double *y_limit;
 y_limit=malloc(sizeof(double)*MAX_NUMBER_OF_STARS);
 x=malloc(sizeof(double)*MAX_NUMBER_OF_STARS);
 y=malloc(sizeof(double)*MAX_NUMBER_OF_STARS);
 data=malloc(sizeof(double)*MAX_NUMBER_OF_STARS);
 char str[256];
 sum=-1;
 int n=0;
 int i=0;
 int n_drop_high_sigma_stars;

 

 dmsf=fopen("data.m_sigma","r");
 if( dmsf==NULL ){
  fprintf(stderr,"ERROR: Can't open data.m_sigma!\n");
  exit(1);
 }

 
 while(-1<fscanf(dmsf,"%lf %lf %lf %lf %s",&m,&sigma,&X,&Y,str)){
  if( sum==-1 ){
   sum=m;
   n=0;
   mmax=m;
  }
  if(m<=mmax+M_SIGMA_BIN_SIZE_M){
   data[n]=sigma;
   n++;
  }
  else{
   sort_data_double(data, 0, n);
   n_drop_high_sigma_stars=MAX((int)(0.07*n),M_SIGMA_BIN_DROP);
   // experimental
   if( n>2*n_drop_high_sigma_stars+2 )
    n-=2*n_drop_high_sigma_stars;
   else
    if( n>n_drop_high_sigma_stars+2 )
     n-=n_drop_high_sigma_stars;
   // end of experimental
   mean=gsl_stats_median_from_sorted_data(data,1,n); //gsl_stats_mean(data,1,n ); // would a median be more appropriate here?
   x[i]=mmax+M_SIGMA_BIN_SIZE_M/2;
   y[i]=mean;
   y_limit[i]=gsl_stats_sd(data,1,n);
   if( 0!=isnan(y_limit[i]) ){
     y_limit[i]=0;
   }
   y_limit[i]=y[i]+M_SIGMA_BIN_MAG_SIGMA_DETECT*y_limit[i]; // now this is the detection limit
   //fprintf(stderr,"%.5lf %.5lf %d\n",x[i],y_limit[i],i);
   i++;
   n=0;
   mmax=m;
  }
 }
 fclose(dmsf);

 // Debug
 //fprintf(stderr,"i=%d\n",i);

 // Interpolate
 gsl_interp_accel *acc = gsl_interp_accel_alloc ();
 gsl_spline *spline = gsl_spline_alloc (gsl_interp_akima, i);
 gsl_spline_init (spline, x, y_limit, i);

 // Compute interpolating function and write it to file
 sigma_selection_curve_log=fopen("vast_sigma_selection_curve.log","w");
 if( sigma_selection_curve_log==NULL ){
  fprintf(stderr,"ERROR: Can't open file vast_sigma_selection_curve.log for writing!\n");
  exit(1);
 }
 double xi,yi;
 for (xi = x[0]; xi < x[i-1]; xi += 0.01){
  yi = gsl_spline_eval (spline, xi, acc);
  fprintf(sigma_selection_curve_log,"%lf %lf\n",xi, yi);
 } 
 fclose(sigma_selection_curve_log);


 // Write stars which pass the selection criteria to data.m_sigma file
 dmsf=fopen("data.m_sigma","r");
 if( dmsf==NULL ){
  fprintf(stderr,"ERROR: Can't open data.m_sigma! (2)\n");
  exit(1);
 }
 while(-1<fscanf(dmsf,"%lf %lf %lf %lf %s",&m,&sigma,&X,&Y,str)){
  // This should avoid "interpolation error" crash.
  if( m<x[0] )m=x[0];
  if( m>x[i-1] )m=x[i-1];
  // Compare the measured value with a limit
  if( sigma>=gsl_spline_eval(spline, m, acc) )
   fprintf(stdout,"%10.6lf %.6lf %9.3lf %9.3lf %s\n",m,sigma,X,Y,str);
 }
 fclose(dmsf);

 return 0;
}
