#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <libgen.h> /* for basename() */

#include <sys/types.h> /* for getpid() */
#include <unistd.h> /* for getpid() too */


#include <getopt.h>

#include <gsl/gsl_statistics_float.h>
#include <gsl/gsl_sort_float.h>

#include "cpgplot.h"

#include "../setenv_local_pgplot.h"

#include "../fitsio.h"

#include "../limits.h"
#include "../vast_math.h"
#include "../photocurve.h"

#include "../ident.h"

void print_pgfv_help(){
 fprintf(stderr,"\n");
 fprintf(stderr,"Usage:\n");
 fprintf(stderr,"press 'I' to get this message.\n");
 fprintf(stderr,"press 'Z' and draw rectangle to zoom in.\n");
 fprintf(stderr,"press 'D' or click middle mouse button to return to the original zoom.\n");
 fprintf(stderr,"press 'H' for Histogram Equalization.\n");
 fprintf(stderr,"press 'B' to invert X axis.\n");
 fprintf(stderr,"press 'V' to invert Y axis.\n");
 fprintf(stderr,"move mouse and press 'F' to adjust image brightness/contrast. If an image apears too bright, move the pointer to the lower left and press 'F'. Repeat it many times to achive the desired result.\n");
 fprintf(stderr,"press 'M' to turn star markers on/off.\n");
 fprintf(stderr,"press 'X' or right click to exit!\nClick on image to get coordinates and value of the current pixel...\n");
 fprintf(stderr,"\n");
 return;
}

/* Magnitude calibration for single image mode */
void magnitude_calibration_using_calib_txt(double *mag, int N){
 int i;
 double a,b,c;
 double a_[4];
 int operation_mode;
 FILE *f;
 /* Check if calib.txt is readable */
 f=fopen("calib.txt","r");
 if( f==NULL )return;
 fclose(f);
 system("lib/fit_mag_calib > calib.tmp");
 f=fopen("calib.tmp","r");
 if( 5==fscanf(f,"%d %lf %lf %lf %lf",&operation_mode,&a_[0],&a_[1],&a_[2],&a_[3]) ){
  // photocurve
  fprintf(stderr,"Calibrating the magnitude scale using the photocurve with parameters:\n%lf %lf %lf %lf\n",a_[0],a_[1],a_[2],a_[3]);
  for(i=0;i<N;i++)
   mag[i]=eval_photocurve( mag[i], a_, operation_mode);
 }
 else{
  // parabola or straight line
  fseek(f,0,SEEK_SET); // go back to the beginning of the file
  fscanf(f,"%lf %lf %lf",&a,&b,&c);
  fprintf(stderr,"Calibrating the magnitude scale using the polynom with parameters:\n%lf %lf %lf\n",a,b,c);
  for(i=0;i<N;i++)
   mag[i]=a*mag[i]*mag[i]+b*mag[i]+c;
 }
 fclose(f);
 system("rm -f calib.tmp");
 return;
}

void update_match_txt(){
 char command_line[2048];
 /* Update PATH variable to make sure the local copy of SExtractor is there */ 
 char pathstring[8192];
 strncpy(pathstring,getenv("PATH"),8192);
 strcat(pathstring,":lib/bin");          
 setenv("PATH", pathstring, 1);
 /* Check if the SExtractor executable (named "sex") is present in $PATH */
 system("lib/look_for_sextractor.sh");                               
 sprintf(command_line,"sex -c default.sex.PHOTO -PARAMETERS_NAME wcs.param wcs.fit");
 fprintf(stderr,"%s\n",command_line);
 system(command_line);
 return;
}

/* Extract reference image name from log file */
void get_ref_image_name(char *str){
 FILE *outfile;
 FILE *datafile;
 char outfilename[2048];
 double MUSOR;
 outfile=fopen("vast_summary.log","r");
 if( outfile==NULL ){
  fprintf(stderr,"ERROR: can't open the log file vast_summary.log\n");
  exit(1);
 }
 fclose(outfile);
 system("grep \"Ref.  image:\" vast_summary.log | awk '{print $6}' > tmp.tmp");
 strcpy(outfilename,"tmp.tmp");
 outfile=fopen(outfilename,"r");
 if( NULL==outfile ){
  fprintf(stderr,"ERROR: Can't open file %s\n",outfilename);
  exit(1);
 }
 fscanf(outfile,"%s",str);
 fclose(outfile);
 system("rm -f tmp.tmp");
 return;
}


void fix_array_with_negative_values( long NUM_OF_PIXELS, float *im){
 long i;
 float min,max;
 min=max=im[0];
 for(i=0;i<NUM_OF_PIXELS;i++){
  if( im[i]<min && im[i]>0 )min=im[i];
  if( im[i]>max && im[i]>0 )max=im[i];
 }
 if( min<0.0 ){
  for(i=0;i<NUM_OF_PIXELS;i++)im[i]=im[i]-min;
  for(i=0;i<NUM_OF_PIXELS;i++){
   if( im[i]>max )max=im[i];
  }
  min=0.001; 
 }
 
 if( max>65535.0 ){
  for(i=0;i<NUM_OF_PIXELS;i++)im[i]=im[i]*65535.0/max;
 }
 max=65535.0;
 
 return;
}

void image_minmax2( long NUM_OF_PIXELS, float *im, float *max_i, float *min_i){
 int i;
 int HIST[65536];
 int summa=0;
 int limit;
 int hist_summa=0;
 (*max_i)=(*min_i)=im[0];
 // set all histogram values to 0 
 for(i=0;i<65536;i++)
  HIST[i]=0;

 for(i=0;i<NUM_OF_PIXELS;i++){
  if( im[i]>0 && im[i]<65535 ){
   HIST[(long)(im[i]+0.5)]+=1;
   if( im[i]>(*max_i) )(*max_i)=im[i];
   if( im[i]<(*min_i) )(*min_i)=im[i];
  }
 }
 for(i=0;i<65535;i++)
  hist_summa+=HIST[i];
 
 limit=(long)( ( (double)hist_summa - (double)hist_summa*PGFV_CUTS_PERCENT/100.0 )/2.0 );

 // find histogram peak
 summa=0;
 for(i=0;i<65535;i++){
  if( summa<HIST[i] ){
   (*min_i)=(float)i;
   summa=HIST[i];
  }
 }
 
 (*min_i)-=(*min_i)*2/3;
 
 (*min_i)=MAX((*min_i),0); // do not go for very negatve values - they are likely wrong
 
 summa=0;
 for(i=65535;i>1;i--){
  summa+=HIST[i];
  
  if( summa>=limit ){
   (*max_i)=(float)i;
   break;
  }
 }
 
 return;
}
  
void image_minmax3( long NUM_OF_PIXELS, float *im, float *max_i, float *min_i, float drawX1, float drawX2, float drawY1, float drawY2, long *naxes){
 long i;
 int HIST[65536];
 int summa=0;
 //long limit;
 int hist_summa=0;
 (*max_i)=(*min_i)=im[0];
 
 float X,Y;
 
 // set all histogram values to 0 
 for(i=0;i<65536;i++)
  HIST[i]=0;

 for(i=0;i<NUM_OF_PIXELS;i++){
  if( im[i]>0 && im[i]<65535 ){
   // Cool it works!!! (Transformation from i to XY)
   Y=1+(int)((float)i/(float)naxes[0]);
   X=i+1-(Y-1)*naxes[0];
   if( X>MIN(drawX1,drawX2) && X<MAX(drawX1,drawX2) && Y>MIN(drawY1,drawY2) && Y<MAX(drawY1,drawY2) ){
    HIST[(long)(im[i]+0.5)]+=1;
    if( im[i]>(*max_i) )(*max_i)=im[i];
    if( im[i]<(*min_i) )(*min_i)=im[i];
   }
  }
 }
 for(i=0;i<65535;i++)
  hist_summa+=HIST[i];
 
 //limit=(long)( ( (double)hist_summa - (double)hist_summa*PGFV_CUTS_PERCENT/100.0 )/2.0 );
 
/*
 // find histogram peak
 summa=0;
 //hist_summa=0;
 for(i=0;i<65535;i++){
  //hist_summa++;
  if( summa<HIST[i] ){
   (*min_i)=(float)i;
   summa=HIST[i];
  }
  //if( hist_summa<(int)(PGFV_CUTS_PERCENT/100.0*(float)(naxes[0]*naxes[1])) )
  // (*max_i)=(float)i;
 }
*/

 summa=0;
 for(i=0;i<65535;i++){
  summa+=HIST[i];
  if( summa>=(int)(PGFV_CUTS_PERCENT/100.0*(float)hist_summa) ){
   (*max_i)=(float)i;
   break;
  }
 }
  

 summa=0;
 for(i=0;i<65535;i++){
  summa+=HIST[i];
  if( summa>=(int)(2.0*(1.0-PGFV_CUTS_PERCENT/100.0)*(float)hist_summa) ){
   (*min_i)=(float)i;
   break;
  }
 }
  
 

 // DEBUG
 //fprintf(stderr,"####### (int)(PGFV_CUTS_PERCENT/100.0*(float)(naxes[0]*naxes[1]))=%d #######\n", (int)(PGFV_CUTS_PERCENT/100.0*(float)(naxes[0]*naxes[1])) );
 
 
 
// (*max_i)=(*min_i)+(*min_i)*2/3;

// (*max_i)=(float)limit;

 (*max_i)=MIN((*max_i),65535); // just in case...
 
 //(*min_i)-=(*min_i)*2/3;
 
 (*min_i)=MAX((*min_i),0); // do not go for very negatve values - they are likely wrong


 // DEBUG
 //fprintf(stderr,"####### max_i=%f min_i=%f #######\n",(*max_i),(*min_i) );

 return;
}


/* 
   Histogram equalization is a method in image processing of contrast adjustment using the image's histogram. 
   See http://en.wikipedia.org/wiki/Histogram_equalization for details.   
*/
void histeq( long NUM_OF_PIXELS, float *im, float *max_i, float *min_i){
 long i;
 int HIST[65536];
 int NO_OF_PIX_BELOW_I[65536];
 (*max_i)=-9999.0;
 (*min_i)=9999.0;
 for(i=0;i<65536;i++){
  HIST[i]=0;
  NO_OF_PIX_BELOW_I[i]=0;
 }
 for(i=0;i<NUM_OF_PIXELS;i++){
  if( im[i]>(*max_i) )
   (*max_i)=im[i];
  if( im[i]<(*max_i) )
   (*min_i)=im[i]; 
 }
 for(i=0;i<NUM_OF_PIXELS;i++){
  HIST[ MAX( 0,(int)( im[i]+0.5 )) ]+=1;
 }
 NO_OF_PIX_BELOW_I[0]=HIST[0];
 for(i=1;i<65536;i++){
   NO_OF_PIX_BELOW_I[i]=NO_OF_PIX_BELOW_I[i-1]+HIST[i];
 }
 for(i=0;i<NUM_OF_PIXELS;i++){
  im[i]=NO_OF_PIX_BELOW_I[ MAX( 0, (int)(im[i]+0.5) ) ]*(*max_i)/NUM_OF_PIXELS;
 }
 return;
}

int myimax(int A, int B){
 if( A>B )
  return A;
 else
  return B;
}

int myimin(int A, int B){
 if( A<B )
  return A;
 else
  return B;
}


int mymax(float A, float B){
 int iA,iB;
 iA=(int)(A+0.5);
 iB=(int)(B+0.5);
 if( iA>iB )
  return iA;
 else
  return iB;
}

int mymin(float A, float B){
 int iA,iB;
 iA=(int)(A+0.5);
 iB=(int)(B+0.5);
 if( iA<iB )
  return iA;
 else
  return iB;
}


int main(int argc, char **argv){
 /* For FITS file reading */
 fitsfile *fptr;       /* pointer to the FITS file; defined in fitsio.h */
 long  fpixel = 1, naxis = 2, nelements, exposure;
 long naxes[2];
 int status = 0, hduT, bitpix, anynul=0,nullval=0;
 unsigned char *image_array_uchar;
 unsigned short *image_array_ushort;
 unsigned int *image_array_uint;
 double *image_array_double;
 float *real_float_array;
 float *float_array;
 float *float_array2;
 long x,y;
 int i;
 /* PGPLOT vars */
 float current_color=0.0;
 float curX,curY,curX2,curY2; 
 char curC='R';
 float tr[6];
 tr[0]=0;
 tr[1]=1;
 tr[2]=0;
 tr[3]=0;
 tr[4]=0;
 tr[5]=1;
 int drawX1,drawX2,drawY1,drawY2,drawX0,drawY0;
 float min_val;
 float max_val;
 
 int hist_trigger=0;
 int mark_trigger=0;

 float markX,markY;

 /* new fatures */
 int buf,v_trigger=0,b_trigger=0;
 int j;
 float axis_ratio;
 double razmer_x,razmer_y;
 
 char fits_image_name[2048];
 int match_mode=0;
 char sex_command_string[2048];
 double APER=0.0;

 
 /* Sex Cat */
 FILE *catfile;
 double MUSOR;
 int intMUSOR;
 int iMUSOR;
 float *sexX;
 float *sexY;
 double *sexFLUX;
 double *sexFLUX_ERR;
 double *sexMAG;
 double *sexMAG_ERR;
 int *sexNUMBER;
 int *sexFLAG;

 double *sexA_IMAGE;
 double *sexERRA_IMAGE;
 double *sexB_IMAGE;
 double *sexERRB_IMAGE;

 int sex=0;
 int marker_counter;
 
 /* Match File */
 FILE *matchfile;
 char RADEC[1024];
 int match_input=0;
 double HH,HM,HS,DD,DM,DS;
 char syscommand[1024];

 /* Calib mode */
 FILE *calibfile; 
 double tmp_APER;
 char imagefilename[1024];
 char system_command[1024];
 int N;
 double catalog_mag;
 
 
 /* For time information from the FITS header */
 double JD; 
 double dimX;
 double dimY; 
 char stderr_output[1024]; 
 char log_output[1024];
 
 int timesys=0; 
 int convert_timesys_to_TT=0;



 int draw_star_markers=1;
 int aperture_change=0;
 
 double median_class_star; 
 double sigma_class_star;

 static float bw_l[] = {-0.5, 0.0, 0.5, 1.0, 1.5, 2.0};
 static float bw_r[] = { 0.0, 0.0, 0.5, 1.0, 1.0, 1.0};
 static float bw_g[] = { 0.0, 0.0, 0.5, 1.0, 1.0, 1.0};
 static float bw_b[] = { 0.0, 0.0, 0.5, 1.0, 1.0, 1.0};

 
 char sextractor_catalog[FILENAME_LENGTH];
 
 int finding_chart_mode=0; // =1 draw finding chart to an image file instead of interactive plotting
 
 if( 0==strcmp("make_finding_chart",basename(argv[0]))){
  fprintf(stderr,"Plotting finding chart...\n");
  finding_chart_mode=1;
 }
              
 /* Reading file which defines rectangular regions we want to exclude */
 float cpgline_tmp_x[2];
 float cpgline_tmp_y[2];
 double X1[500], Y1[500], X2[500], Y2[500];
 int N_bad_regions = 0;
 read_bad_lst(X1, Y1, X2, Y2, &N_bad_regions);

 int use_ds9=0; // if 1 - use ds9 instead of pgplot to display an image
 pid_t pid=getpid();
 char ds9_region_filename[1024];

 if( argc==1 ){
  fprintf(stderr,"Usage:\n%s FITSIMAGE.fit\nor\n%s FITSIMAGE.fit X Y\nor\n%s match\n",argv[0],argv[0],argv[0]);
  exit(1);
 }


 double fixed_aperture=0.0;
 /* Options for getopt() */ 
 char *cvalue = NULL;
         
 const char* const shortopt = "a:9";
 const struct option longopt[] = {
         {"apeture", 1, NULL, 'a'}, {"ds9", 0, NULL, '9'}, {NULL, 0, NULL, 0}
 }; //NULL string must be in the end
 int nextopt;
 while (nextopt = getopt_long(argc, argv, shortopt, longopt, NULL), nextopt != -1) {
  switch (nextopt) {
   case 'a':
    cvalue = optarg;
    fixed_aperture=atof(cvalue);
    fprintf(stdout, "opt 'a': Using fixed aperture %.1lf pix. in diameter!\n",fixed_aperture);
    break;
   case '?':
    if (optopt == 'a'){
     fprintf (stderr, "Option -%c requires an argument: fixed aperture size in pix.!\n", optopt);
     exit(1);
    }
    break;
   case '9':
    use_ds9=1;
    fprintf(stdout, "opt '9': Using ds9 to display images!\n");
    break;
   case  -1:
    fprintf(stderr, "That's all\n");
    break;
  }
 }
 optind--; //!!!

 if( argc-optind>=4 ){
  mark_trigger=1;
  markX=(float)atof(argv[optind+2]);
  markY=(float)atof(argv[optind+3]);
  //fprintf(stderr,"Putting mark on pixel position %lf %lf\n",markX,markY);
 }
 if( argc-optind==5 ){
  APER=atof(argv[optind+4]);
 }
 
 strcpy(fits_image_name,argv[optind+1]);

 if( 0==strcasecmp(fits_image_name,"calib") )match_mode=2; /* Magnitude calibration mode */
 if( match_mode==2 ){
  /* Remove old calib.txt */
  system("rm -f calib.txt");
 
  /* Allocate memory for the arrays */
  sexX=(float *)malloc(MAX_NUMBER_OF_STARS*sizeof(float));
  sexY=(float *)malloc(MAX_NUMBER_OF_STARS*sizeof(float));
  sexMAG=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexMAG_ERR=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  marker_counter=0;
  
  /* Get reference file name from log */
  get_ref_image_name(fits_image_name);
  
  /* Read data.m_sigma but select only stars detected on the reference frame */
  matchfile=fopen("data.m_sigma","r");
  while(-1<fscanf(matchfile,"%lf %lf %f %f %s",&sexMAG[sex],&sexMAG_ERR[sex],&sexX[sex],&sexY[sex],RADEC)){
   calibfile=fopen(RADEC,"r");
   if( calibfile!=NULL ){
    fscanf(calibfile,"%lf %lf %lf %lf %lf %lf %s",&MUSOR,&MUSOR,&MUSOR,&MUSOR,&MUSOR,&tmp_APER,imagefilename);
    fclose(calibfile);
    if( 0==strcmp(imagefilename,fits_image_name) ){
     /* Get number of observations for correct error estimation */
     sprintf(system_command,"grep -c \" \" %s > grep.tmp",RADEC);
     system(system_command);
     calibfile=fopen("grep.tmp","r");
     fscanf(calibfile,"%d",&N);
     fclose(calibfile);
     system("rm -f grep.tmp");
     sexMAG_ERR[sex]=sexMAG_ERR[sex]/sqrt(N-1);
     /* done with errors, remember aperture size, increase counter */
     APER=tmp_APER;
     sex++;
    }
   }
  }
  fclose(matchfile);
  sex--; /* We can't be sure that the last star is visible on the reference frame so we just drop it */  
 }
 
 
 if( 0==strcasecmp(fits_image_name,"match") ){
  match_mode=1;
  system("rm -f match.txt");
  get_ref_image_name(fits_image_name);
 }


 if( 0==strcmp("sextract_single_image",basename(argv[0])) ){
  match_mode=3;
 }
 
 
 if( 0==strcasecmp(fits_image_name,"detect") ){
  if( argc-optind<3 ){
   fprintf(stderr,"Usage: ./pgfv detect image.fit\n");
   exit(1);
  }
  strcpy(fits_image_name,argv[optind+2]);
  match_mode=3;
 }
 if( match_mode==3 ){
  fprintf(stderr,"Entering single image reduction mode.\nProcessing image %s\n",fits_image_name);
  fprintf(stderr,"Use '+' or '-' to increase or decrease aperture size.\n");
  fprintf(stderr,"\E[34;47mTo calibrate magnitude scale press '2'\E[33;00m\n");

  /* Remove old calib.txt in case we'll want a magnitude calibration */
  system("rm -f calib.txt");
 }

 /* Get time and frame size information from the FITS header */
 if( 0!=fitsfile_read_check(fits_image_name) )exit(1);
 int param_nojdkeyword=0; // Temporary fix!!! pgfv cannot accept the --nojdkeyword parameter yet, only the main program vast understands it
 gettime(fits_image_name, &JD, &timesys, convert_timesys_to_TT, &dimX, &dimY, stderr_output, log_output, param_nojdkeyword, 0);
 stderr_output[strlen(stderr_output)-1]='\0'; /* Remove \n at the end of line */


 if( match_mode==1 || match_mode==3 ){
  /* Check if the SExtractor executable (named "sex") is present in $PATH */
  /* Update PATH variable to make sure the local copy of SExtractor is there */ 
  char pathstring[8192];
  strncpy(pathstring,getenv("PATH"),8192);
  strcat(pathstring,":lib/bin");          
  setenv("PATH", pathstring, 1);
  system("lib/look_for_sextractor.sh");
  /* Star match mode (create WCS) or Single image reduction mode */
  APER=autodetect_aperture(fits_image_name, sextractor_catalog, 0, 0, fixed_aperture, 1, dimX, dimY);
  if( fixed_aperture!=0.0 )APER=fixed_aperture;
  
  sexX=(float *)malloc(MAX_NUMBER_OF_STARS*sizeof(float));
  sexY=(float *)malloc(MAX_NUMBER_OF_STARS*sizeof(float));
  sexFLUX=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexFLUX_ERR=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexMAG=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexMAG_ERR=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexNUMBER=(int *)malloc(MAX_NUMBER_OF_STARS*sizeof(int));
  sexFLAG=(int *)malloc(MAX_NUMBER_OF_STARS*sizeof(int));

  sexA_IMAGE=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexERRA_IMAGE=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexB_IMAGE=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  sexERRB_IMAGE=(double *)malloc(MAX_NUMBER_OF_STARS*sizeof(double));
  
  catfile=fopen(sextractor_catalog,"r");
  if( NULL==catfile ){fprintf(stderr,"ERROR! Cannot open sextractor catalog file %s for reading!\n",sextractor_catalog);exit(1);}
  while( -1<fscanf(catfile, "%d %lf %lf %lf %lf %f %f %lf %lf %lf %lf %d\n", &sexNUMBER[sex], &sexFLUX[sex], &sexFLUX_ERR[sex], &sexMAG[sex], &sexMAG_ERR[sex], &sexX[sex], &sexY[sex], &sexA_IMAGE[sex], &sexERRA_IMAGE[sex], &sexB_IMAGE[sex], &sexERRB_IMAGE[sex], &sexFLAG[sex]) ){
   sex++;
  }
  fclose(catfile);

  // if we use ds9 to display an image
  if( use_ds9==1 ){
   // prepare the ds9 region file
   sprintf(ds9_region_filename,"ds9_%d_tmp.reg",pid);
   catfile=fopen(ds9_region_filename,"w");
   fprintf(catfile,"# Region file format: DS9 version 4.0\n# Filename: %s\nglobal color=green font=\"sans 8 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source\nimage\n",fits_image_name);
   for(;sex--;)fprintf(catfile,"circle(%.3lf,%.3lf,%.1lf)\n# text(%.3lf,%.3lf) text={%d}\n",sexX[sex],sexY[sex],APER/2.0,sexX[sex],sexY[sex]-APER,sexNUMBER[sex]);
   fclose(catfile);
  
   // execute the system command to run ds9
   fprintf(stderr,"Starting DS9 FITS image viewer...\n");
   sprintf(stderr_output,"ds9 %s -region %s -xpa no ; rm -f %s\n",fits_image_name,ds9_region_filename,ds9_region_filename);
   fprintf(stderr,"%s",stderr_output);
   system(stderr_output);
   
   // free the arrays
   free(sexX);
   free(sexY);
   free(sexFLUX);
   free(sexFLUX_ERR);
   free(sexMAG);
   free(sexMAG_ERR);
   free(sexNUMBER);
   free(sexFLAG);
   free(sexA_IMAGE);
   free(sexERRA_IMAGE);
   free(sexB_IMAGE);
   free(sexERRB_IMAGE);
   // exit
   return 1;
  }
  

 }
 
 if( 0!=fitsfile_read_check(fits_image_name) )exit(1);    
 ffopen(&fptr, fits_image_name, 0 , &status);
 fits_get_img_type(fptr,&bitpix,&status);
 fits_read_key(fptr,TLONG,"NAXIS1",&naxes[0],NULL,&status);
 fits_read_key(fptr,TLONG,"NAXIS2",&naxes[1],NULL,&status);
 fprintf(stderr,"Image: %ldx%ld pixels, BITPIX data type code: %d\n",naxes[0],naxes[1],bitpix);
 float_array=malloc(naxes[0] * naxes[1] * sizeof(float));
 real_float_array=malloc(naxes[0] * naxes[1] * sizeof(float));

 // 8 bit image
 if( bitpix==8 ){
  image_array_uchar=(unsigned char *)malloc(naxes[0] * naxes[1] * sizeof(unsigned char));
  fits_read_img(fptr, TBYTE, 1, naxes[0]*naxes[1],&nullval,image_array_uchar, &anynul, &status);
  for(i=0;i<naxes[0]*naxes[1];i++)float_array[i]=(float)image_array_uchar[i];
 } 
 // 16 bit image
 if( bitpix==16 ){
  image_array_ushort=(unsigned short *)malloc(naxes[0] * naxes[1] * sizeof(short));
  fits_read_img(fptr, TUSHORT, 1, naxes[0]*naxes[1],&nullval,image_array_ushort, &anynul, &status);
  if( status==412 ){
   fits_report_error(stderr, status);  /* print out any error messages */
   fprintf(stderr,"Image read problem! Is it actually a float-type image? Let's try...\n");
   status=0;
   bitpix=-32;
  }
  for(i=0;i<naxes[0]*naxes[1];i++)float_array[i]=(float)image_array_ushort[i];
  free(image_array_ushort);
 }
 // 32 bit image
 if( bitpix==32 ){
  image_array_uint=(unsigned int *)malloc(naxes[0] * naxes[1] * sizeof(int));
  fits_read_img(fptr, TUINT, 1, naxes[0]*naxes[1],&nullval,image_array_uint, &anynul, &status);
  for(i=0;i<naxes[0]*naxes[1];i++)float_array[i]=(float)image_array_uint[i];
  free(image_array_uint);
 }
 // double image
 if( bitpix==-64 ){
  image_array_double=(double *)malloc(naxes[0] * naxes[1] * sizeof(double));
  fits_read_img(fptr, TDOUBLE, 1, naxes[0]*naxes[1],&nullval,image_array_double, &anynul, &status);
  for(i=0;i<naxes[0]*naxes[1];i++)float_array[i]=(float)image_array_double[i];
  free(image_array_double);
 }
 // float image
 if( bitpix==-32 ){
  fits_read_img(fptr, TFLOAT, 1, naxes[0]*naxes[1],&nullval,float_array, &anynul, &status);
 }
 ffclos (fptr, &status);
 fits_report_error(stderr, status);  /* print out any error messages */
 if( status!=0 )
  exit(status);
 //fprintf(stderr,"Ok\n");

 axis_ratio=(float)naxes[0]/(float)naxes[1];
 //fprintf(stderr,"axis_ratio=%f\n",axis_ratio);


 // real_float_array - array with real pixel values (well, not real but converted to float)
 // float_array - array used for computations with values ranging from 0 to 65535
 for(i=0;i<naxes[0]*naxes[1];i++)real_float_array[i]=float_array[i];
 fix_array_with_negative_values( naxes[0]*naxes[1], float_array);
 image_minmax2( naxes[0]*naxes[1], float_array, &max_val, &min_val);

 /* GUI */
 //fprintf(stderr,"Opening display ... ");

 //setenv("PGPLOT_DIR","lib/pgplot/",1);
 setenv_localpgplot(argv[0]);
 if( finding_chart_mode==1 ){
  if(cpgbeg(0, "/PNG", 1, 1) != 1)
   return EXIT_FAILURE;
 }
 else{
  if(cpgbeg(0, "/XW", 1, 1) != 1)
   return EXIT_FAILURE;
 }

 //cpgpap( 0.0, 1.0); /* Make square plot */
 
 //if( finding_chart_mode==0 ){ 
  cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
  cpgpage();
 //}
 if( finding_chart_mode==0 ){
  cpgpap( 0.0, (float)naxes[1]/(float)(naxes[0]) );
  cpgsvp(0.05, 0.95, 0.035, 0.035+0.9);
 }
 else{
  cpgpap( 0.0, 1.0); /* Make square plot */
  cpgsvp(0.05, 0.95, 0.05, 0.95);
 }

 /* set default plotting limits */
 if( finding_chart_mode==1 ){
  /* Set plotting limits for a finding chart */  
  drawX1=1;drawX2=naxes[0];drawY1=1;drawY2=naxes[1];
 }
 else
 {
  /* Start with inverted Y */
  drawX1=1;
  drawY1=naxes[1];
  drawX2=naxes[0];
  drawY2=1;
 }
 //fprintf(stderr,"Ok\n");

 if( finding_chart_mode==0 )
  fprintf(stderr,"\n[%s] Press 'I' if you need help...\n\n",argv[0]);
 
 if( finding_chart_mode==1 ){
  curX=markX;
  curY=markY;
 }
// else
  curC='R';
 do{
  
  if( curX>0 && curX<naxes[0] && curY>0 && curY<naxes[1] ){

   /* '+' increse aperture */
   if( curC=='+' || curC=='=' ){
    APER+=0.5;
    aperture_change=1;
   }

   /* '-' decrese aperture */
   if( curC=='-' || curC=='_' ){
    APER-=0.5;
    aperture_change=1;
   }
   
   /* If aperture was changed - repeat measurements with new aperture */
   if( aperture_change==1 && match_mode==3 ){
    autodetect_aperture(fits_image_name, sextractor_catalog, 1, 0, APER, 1, dimX, dimY);
    sex=0;
    catfile=fopen(sextractor_catalog,"r");
    if( NULL==catfile ){fprintf(stderr,"ERROR! Cannot open sextractor catalog file %s for reading!\n",sextractor_catalog);exit(1);}
    while( -1<fscanf(catfile, "%d %lf %lf %lf %lf %f %f %lf %lf %lf %lf %d\n", &sexNUMBER[sex], &sexFLUX[sex], &sexFLUX_ERR[sex], &sexMAG[sex], &sexMAG_ERR[sex], &sexX[sex], &sexY[sex], &sexA_IMAGE[sex], &sexERRA_IMAGE[sex], &sexB_IMAGE[sex], &sexERRB_IMAGE[sex], &sexFLAG[sex]) ){
     sex++;
    }
    fclose(catfile);
    fprintf(stderr,"New aperture %.1lf\n",APER);
    aperture_change=0;
    curC='R'; // Redraw screen
   }
   
   /* Switch to magnitude calibration mode */
   if( curC=='2' && match_mode==3 ){
    fprintf(stderr,"Entering megnitude calibration mode!\n");
    fprintf(stderr,"\E[01;31mPlease click on comparison stars and enter their magnitudes...\E[33;00m\n");
    fprintf(stderr,"\E[01;31mPress '3' when done!\E[33;00m\n");
    system("rm -f calib.txt");
    match_mode=2;
   }
   
   /* Switch to single image inspection mode */
   if( curC=='3' && match_mode==2 ){
    magnitude_calibration_using_calib_txt(sexMAG, sex);
    fprintf(stderr,"Entering single image inspection mode!\n");
    match_mode=3;
   }
   
   
   /* I - print info (help) */
   if( curC=='I' || curC=='i' )
    print_pgfv_help();

   /* M - star markers on/off */
   if( curC=='M' || curC=='m' ){
    if( draw_star_markers==1 )
     draw_star_markers=0;
    else
     draw_star_markers=1;
    curC='R'; 
   }

   /* Process left mouse button click */
   if( curC=='A' ){
    fprintf(stderr,"Pixel: %7.1f %7.1f %9.3f\n",curX+0.5,curY+0.5,real_float_array[(int)(curX-0.5)+(int)(curY-0.5)*naxes[0]]);
    
    /* Star match mode (create WCS) */
    if( match_mode==1 ){
     for(marker_counter=0;marker_counter<sex;marker_counter++){
      if( (curX-sexX[marker_counter])*(curX-sexX[marker_counter])+(curY-sexY[marker_counter])*(curY-sexY[marker_counter])<(float)(APER*APER) ){
       fprintf(stderr,"Coordinates: ???\n");
       fgets(RADEC,1024,stdin);
       sscanf(RADEC,"%lf:%lf:%lf %lf:%lf:%lf",&HH,&HM,&HS,&DD,&DM,&DS);
       HH+=HM/60+HS/3600;
       HH*=15;
       if( DD>0 )DD+=DM/60+DS/3600;
       if( DD<0 )DD-=DM/60+DS/3600;
       fprintf(stderr,"%.3f %.3f %12.7lf %+12.7lf\n",sexX[marker_counter], sexY[marker_counter], HH, DD);
       matchfile=fopen("match.txt","a");
       fprintf(matchfile,"%.3f %.3f %12.7lf %+12.7lf\n",sexX[marker_counter], sexY[marker_counter], HH, DD);
       fclose(matchfile);
       match_input++;
       break;
      }
     }
    }
    
    /* Magnitude calibration mode or Single image mode */
    if( match_mode==2 || match_mode==3 ){
     for(marker_counter=0;marker_counter<sex;marker_counter++){
      if( (curX-sexX[marker_counter])*(curX-sexX[marker_counter])+(curY-sexY[marker_counter])*(curY-sexY[marker_counter])<(float)(APER*APER) ){
       
       /* Magnitude calibration mode */
       if( match_mode==2 ){
        fprintf(stderr,"Star detected. Instrumental magnitude: %.4lf %.4lf\nPlease, enter it's catalog magnitude: ",sexMAG[marker_counter],sexMAG_ERR[marker_counter]);
        fgets(RADEC,1024,stdin);
        catalog_mag=atof(RADEC);
        matchfile=fopen("calib.txt","a");
        fprintf(matchfile,"%.4lf %.4lf %.4lf\n",sexMAG[marker_counter], catalog_mag, sexMAG_ERR[marker_counter]);
        fclose(matchfile);
        match_input++;
        break;
       }
       
       /* Single image mode */
       if( match_mode==3 ){
        fprintf(stderr,"Star %6d\n",sexNUMBER[marker_counter]);
        
        if( sexX[marker_counter] > OTSTUP && sexY[marker_counter] > OTSTUP && fabs(sexX[marker_counter] - (float)naxes[0]) > OTSTUP && fabs(sexY[marker_counter] - (float)naxes[1]) > OTSTUP )
         fprintf(stderr,"Star coordinates \E[01;32m%6.1lf %6.1lf\E[33;00m (pix)\n",sexX[marker_counter],sexY[marker_counter]);
        else
         fprintf(stderr,"Star coordinates \E[01;31m%6.1lf %6.1lf\E[33;00m (pix)\n",sexX[marker_counter],sexY[marker_counter]);

        if( 0==exclude_region( X1, Y1, X2, Y2, N_bad_regions, (double)sexX[marker_counter], (double)sexY[marker_counter], APER) )
         fprintf(stderr,"The star is not situated in a bad CCD region according to bad_region.lst\n");
        else
         fprintf(stderr,"The star is not situated in a \E[01;31mbad CCD region\E[33;00m according to bad_region.lst\n");
         
        if( sexFLUX[marker_counter]>MIN_SNR*sexFLUX_ERR[marker_counter] )
         fprintf(stderr,"SNR \E[01;32m%lf\E[33;00m\n",sexFLUX[marker_counter]/sexFLUX_ERR[marker_counter]);
        else
         fprintf(stderr,"SNR \E[01;31m%lf\E[33;00m\n",sexFLUX[marker_counter]/sexFLUX_ERR[marker_counter]);

        if( sexMAG[marker_counter]!=99.0000 )
         fprintf(stderr,"Magnitude \E[01;34m%7.4lf  %6.4lf\E[33;00m\n",sexMAG[marker_counter],sexMAG_ERR[marker_counter]);
        else
         fprintf(stderr,"Magnitude \E[01;31m%7.4lf  %6.4lf\E[33;00m\n",sexMAG[marker_counter],sexMAG_ERR[marker_counter]);

        if( sexFLAG[marker_counter]==0 )
         fprintf(stderr,"SExtractor flag \E[01;32m%d\E[33;00m\n",sexFLAG[marker_counter]);
        else
         fprintf(stderr,"SExtractor flag \E[01;31m%d\E[33;00m\n",sexFLAG[marker_counter]);

        if( sexA_IMAGE[marker_counter]+sexB_IMAGE[marker_counter]+3*sqrt(sexERRA_IMAGE[marker_counter]*sexERRA_IMAGE[marker_counter]+sexERRB_IMAGE[marker_counter]*sexERRB_IMAGE[marker_counter])>2*MIN( FWHM_MIN, APER/(CONST+1) ) )
         fprintf(stderr,"A= \E[01;32m%lf %lf  B= %lf %lf\E[33;00m\n",sexA_IMAGE[marker_counter],sexERRA_IMAGE[marker_counter],sexB_IMAGE[marker_counter],sexERRB_IMAGE[marker_counter]);
        else
         fprintf(stderr,"A= \E[01;31m%lf %lf  B= %lf %lf\E[33;00m\n",sexA_IMAGE[marker_counter],sexERRA_IMAGE[marker_counter],sexB_IMAGE[marker_counter],sexERRB_IMAGE[marker_counter]);
        
        /* 
        if( sexSTARGAL[marker_counter]<MAX_CLASS_STAR )
         fprintf(stderr,"CLASS_STAR= \E[01;32m%.2lf\E[33;00m   median= %.2lf sigma= %.2lf\n",sexSTARGAL[marker_counter],median_class_star,sigma_class_star);
        else
         fprintf(stderr,"CLASS_STAR= \E[01;31m%.2lf\E[33;00m   median= %.2lf sigma= %.2lf\n",sexSTARGAL[marker_counter],median_class_star,sigma_class_star);
        */

        fprintf(stderr,"%s\n",stderr_output);
        fprintf(stderr,"\n");
       }
       
      }
     }
    }
   } // if( curC=='A' ){

   if( finding_chart_mode==1 )
    curC='Z';
   
   /* Zoom in or out */
   if( curC=='z' || curC=='Z' ){
    if( finding_chart_mode==1 ){
     drawX1=markX-10;
     drawX2=markX+10;
     drawY1=markY-10;
     drawY2=markY+10;
     curC='R';
    }
    else{
     cpgsci(5);
     cpgband( 2, 0, curX, curY, &curX2, &curY2, &curC);
     cpgsci(1);
    }
    if( curC=='Z' || curC=='z' )
     curC='D';
    else{
     if( finding_chart_mode==0 ){
      drawX1=mymin(curX,curX2);
      drawX2=mymax(curX,curX2);
      drawY1=mymin(curY,curY2);
      drawY2=mymax(curY,curY2);
     }
     drawX0=(int)((drawX1+drawX2)/2+0.5);
     drawY0=(int)((drawY1+drawY2)/2+0.5);
     razmer_y=myimax(drawX2-drawX1,drawY2-drawY1);
     if( finding_chart_mode==1 ){
      // we whant a square finding chart !
      razmer_x=razmer_y;
     }
     else{
      razmer_x=axis_ratio*razmer_y;
     }
     drawX1=drawX0-(int)(razmer_x/2+0.5);
     drawY1=drawY0-(int)(razmer_y/2+0.5);
     drawX2=drawX1+(int)razmer_x;
     drawY2=drawY1+(int)razmer_y;
     if( drawX2>naxes[0] ){
      drawX1-=drawX2-naxes[0];
      drawX2=naxes[0];
     }
     if( drawY2>naxes[1] ){
      drawY1-=drawY2-naxes[1];
      drawY2=naxes[1];
     }
     if( drawX1<1 ){
      drawX2+=1-drawX1;
      drawX1=1;
     }
     if( drawY1<1 ){
      drawY2+=1-drawY1;
      drawY1=1;
     }
     if( drawX2>naxes[0] )
      drawX2=naxes[0];
     if( drawY2>naxes[1] )
      drawY2=naxes[1];
     buf=drawY1;
     drawY1=drawY2;
     drawY2=buf;
     curC='R';
    }
   }
  }
  //if( curC=='x' )curC='X'; // Oh, it seems this doesn't work
  if( curC=='H' || curC=='h' ){
   if( hist_trigger==0 ){
    hist_trigger=1;
    float_array2=malloc(naxes[0]*naxes[1]*sizeof(float));
    for(i=0;i<naxes[0]*naxes[1];i++){
     float_array2[i]=float_array[i];
    }
    //fprintf(stderr,"histeq... ");
    histeq( naxes[0]*naxes[1], float_array, &max_val, &min_val);
    image_minmax3( naxes[0]*naxes[1], float_array, &max_val, &min_val, drawX1, drawX2, drawY1, drawY2, naxes); // TEST
    //fprintf(stderr,"Ok\n ");
   }
   else{
    hist_trigger=0;
    for(i=0;i<naxes[0]*naxes[1];i++){
     float_array[i]=float_array2[i];
    }
    free(float_array2);
    //image_minmax2( (int)(naxes[0]*naxes[1]), float_array, &max_val, &min_val);
    image_minmax3( naxes[0]*naxes[1], float_array, &max_val, &min_val, drawX1, drawX2, drawY1, drawY2, naxes);
   }
   curC='R';
  }
  if( curC=='D' || curC=='d' ){
   drawX1=1;
   drawY1=naxes[1];
   drawX2=naxes[0];
   drawY2=1;
   curC='R';
  }
  if( curC=='V' || curC=='v' ){
   buf=drawY1;
   drawY1=drawY2;
   drawY2=buf;
   curC='R';
  }
  if( curC=='B' || curC=='b' ){
   buf=drawX1;
   drawX1=drawX2;
   drawX2=buf;
   curC='R';
  }
  /* F - Fiddle the color table contrast and brightness */
  if( curC=='F' || curC=='f' ){
   fprintf(stderr,"brightness=%lf  contrast=%lf\n",(curX - drawX1)/(drawX2 - drawX1), 5.0 * curY / fabs(drawY2-drawY1));
   cpgctab( bw_l, bw_r, bw_g, bw_b, 83, 5.0 * curY / fabs(drawY2-drawY1), (curX - drawX1)/(drawX2 - drawX1));
   curC='R'; 
  }
  
  /* R - Redraw screen */
  if( curC=='R' || curC=='r' ){
  
   if( finding_chart_mode==0 ){
    cpgbbuf();
    cpgscr( 0, 0.10, 0.31, 0.32);  /* set default vast window background */
    cpgeras();
   }
   cpgswin( (float)drawX1, (float)drawX2, (float)drawY1, (float)drawY2);
   cpgbox("BCN1", 0.0 , 0, "BCN1", 0.0, 0);
   if( drawY1>drawY2 ){
    buf=drawY1;
    drawY1=drawY2;
    drawY2=buf;
    v_trigger=1;       
   }
   if( drawX1>drawX2 ){
    buf=drawX1;
    drawX1=drawX2;
    drawX2=buf;
    b_trigger=1;       
   }

   // Determine cuts
   image_minmax3( naxes[0]*naxes[1], float_array, &max_val, &min_val, drawX1, drawX2, drawY1, drawY2, naxes);

   /* Draw image */
   if( finding_chart_mode==0 ){
    cpgscr( 0, 0.0, 0.0, 0.0);  /* set black background */   
    cpgimag( float_array, (int)naxes[0], (int)naxes[1], drawX1, drawX2, drawY1, drawY2, min_val, max_val, tr);
   }
   else{
    //fprintf(stderr,"curC=%c\n",curC);
    cpgscr( 1, 0.0, 0.0, 0.0);
    cpgscr( 0, 1.0, 1.0, 1.0);
    cpggray( float_array, (int)naxes[0], (int)naxes[1], drawX1, drawX2, drawY1, drawY2, min_val, max_val, tr);
    cpgscr( 0, 0.0, 0.0, 0.0);
    cpgscr( 1, 1.0, 1.0, 1.0);
//    cpgclos();
//    return 0;
   }
   /* Make labels with general information: time, filename... */
   if( finding_chart_mode==0 ){
    cpgscr( 1, 0.62, 0.81, 0.38); /* set color of lables */
    cpgsch(0.9); /* Set small font size */
    cpgmtxt("T",0.5,0.5,0.5,argv[optind+1]);
    cpgmtxt("T",1.5,0.5,0.5,stderr_output);
    cpgsch(1.0); /* Set normal font size */
    cpgscr( 1, 1.0, 1.0, 1.0); /* */
   }
   else{
    cpgmtxt("T",1.0,0.5,0.5,stderr_output);
   }
    /* Done with labels */
    
   /* Put a mark */ 
   if( mark_trigger==1 ){
    cpgsci(2);
    cpgpt1( markX, markY, 2);
    cpgsci(1);
   }
   
   if( finding_chart_mode==1 ){
    // exit now
    cpgclos();
    return 0;
   }

   /* If not in simple display mode - draw star markers */
   if( match_mode>0 && draw_star_markers==1 ){
    cpgsci(3);
    cpgsfs(2);
    for(marker_counter=0;marker_counter<sex;marker_counter++)cpgcirc(sexX[marker_counter], sexY[marker_counter], (float)APER/2.0);
    /* And draw bad regions */
    if( 0!=N_bad_regions ){
     cpgsci(2);
     for(marker_counter=0;marker_counter<N_bad_regions;marker_counter++){
      cpgline_tmp_x[0]=(float)X1[marker_counter];
      cpgline_tmp_y[0]=(float)Y1[marker_counter];
      cpgline_tmp_x[1]=(float)X1[marker_counter];
      cpgline_tmp_y[1]=(float)Y2[marker_counter];
      cpgline(2, cpgline_tmp_x, cpgline_tmp_y);
      
      cpgline_tmp_x[0]=(float)X1[marker_counter];
      cpgline_tmp_y[0]=(float)Y2[marker_counter];
      cpgline_tmp_x[1]=(float)X2[marker_counter];
      cpgline_tmp_y[1]=(float)Y2[marker_counter];
      cpgline(2, cpgline_tmp_x, cpgline_tmp_y);
      
      cpgline_tmp_x[0]=(float)X2[marker_counter];
      cpgline_tmp_y[0]=(float)Y2[marker_counter];
      cpgline_tmp_x[1]=(float)X2[marker_counter];
      cpgline_tmp_y[1]=(float)Y1[marker_counter];
      cpgline(2, cpgline_tmp_x, cpgline_tmp_y);
      
      cpgline_tmp_x[0]=(float)X2[marker_counter];
      cpgline_tmp_y[0]=(float)Y1[marker_counter];
      cpgline_tmp_x[1]=(float)X1[marker_counter];
      cpgline_tmp_y[1]=(float)Y1[marker_counter];
      cpgline(2, cpgline_tmp_x, cpgline_tmp_y);
     }
    }
    cpgsci(1);
   }
   /* Else - draw single star marker */
   if( match_mode==0 && APER>0 ){
    cpgsci(2);
    cpgsfs(2);
    cpgcirc(markX, markY, (float)APER/2.0);
    cpgsci(1);
   }
  
   //fprintf(stderr,"finding_chart_mode=%d\n",finding_chart_mode); 
   if( finding_chart_mode==0 )
    cpgebuf();
   else{
    cpgclos();
    return 0;
   }
   
   
   if( v_trigger==1 ){
    buf=drawY1;
    drawY1=drawY2;
    drawY2=buf;
    v_trigger=0;
   }
   if( b_trigger==1 ){
    buf=drawX1;
    drawX1=drawX2;
    drawX2=buf;
    b_trigger=0;          
   }
  }
  
  cpgcurs(&curX, &curY, &curC);
 }while( curC!='X' && curC!='x' );

 if( match_mode>0 ){
  free(sexX);
  free(sexY);
  free(sexMAG);
  free(sexMAG_ERR);
 }
 
 if( match_mode==1 || match_mode==3 ){
  free(sexFLUX);
  free(sexFLUX_ERR);
  free(sexNUMBER);
  free(sexFLAG);
  //free(sexSTARGAL);
  free(sexA_IMAGE);
  free(sexERRA_IMAGE);
  free(sexB_IMAGE);
  free(sexERRB_IMAGE);
 }
 
 /* Write WCS header */
 /* Star match mode (create WCS) */
 if( match_mode==1 ){
  sprintf(syscommand,"cp %s wcs.fit",fits_image_name);
  fprintf(stderr,"%s\n",syscommand);
  system(syscommand);
  sprintf(syscommand,"util/match_eater wcs.fit");
  fprintf(stderr,"%s\n",syscommand);
  system(syscommand);
  fprintf(stderr,"Reference image is written to wcs.fit\n");
 }
 
 /* Write magnitude calibration file */
 /* Magnitude calibration mode */
 if( match_mode==2 && match_input!=0 ){
  fprintf(stderr,"%d stars were written to calib.txt \n", match_input);
 }
 
 if( hist_trigger==1 )free(float_array2);
 
 free(float_array);
 free(real_float_array);
 
 return 0;
}
