#include <stdio.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_errno.h>

int wpolyfit(double *datax, double *datay, double *dataerr, int n, double *poly_coeff) {


	gsl_set_error_handler_off();

	int i;
	double xi, yi, ei, chisq;
	gsl_matrix *X, *cov;
	gsl_vector *y, *w, *c;

	X = gsl_matrix_alloc(n, 3);
	y = gsl_vector_alloc(n);
	w = gsl_vector_alloc(n);

	c = gsl_vector_alloc(3);
	cov = gsl_matrix_alloc(3, 3);

	for (i = 0; i < n; i++) {
		xi = datax[i];
		yi = datay[i];
		ei = dataerr[i];

		gsl_matrix_set(X, i, 0, 1.0);
		gsl_matrix_set(X, i, 1, xi);
		gsl_matrix_set(X, i, 2, xi*xi);

		gsl_vector_set(y, i, yi);
		gsl_vector_set(w, i, 1.0 / (ei*ei));
	}

		gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc(n, 3);
		if( 0!=gsl_multifit_wlinear(X, w, y, c, cov, &chisq, work) )return 1;
		gsl_multifit_linear_free(work);

#define C(i) (gsl_vector_get(c,(i)))
#define COV(i,j) (gsl_matrix_get(cov,(i),(j)))

		poly_coeff[0] = C(0);
		poly_coeff[1] = C(1);
		poly_coeff[2] = C(2);
		poly_coeff[5] = COV(0, 0);
		poly_coeff[6] = COV(1, 1);
		poly_coeff[7] = COV(2, 2);

        /* Free GSL stuff */
        gsl_matrix_free(X);
        gsl_vector_free(y);
        gsl_vector_free(w);
        gsl_vector_free(c);
        gsl_matrix_free(cov);

	return 0;
}

int wlinearfit(double *datax, double *datay, double *dataerr, int n, double *poly_coeff) {


	gsl_set_error_handler_off();

	int i;
	double xi, yi, ei, chisq;
	gsl_matrix *X, *cov;
	gsl_vector *y, *w, *c;

	X = gsl_matrix_alloc(n, 2);
	y = gsl_vector_alloc(n);
	w = gsl_vector_alloc(n);

	c = gsl_vector_alloc(2);
	cov = gsl_matrix_alloc(2, 2);

	for (i = 0; i < n; i++) {
		xi = datax[i];
		yi = datay[i];
		ei = dataerr[i];

		gsl_matrix_set(X, i, 0, 1.0);
		gsl_matrix_set(X, i, 1, xi);
		//gsl_matrix_set(X, i, 2, xi*xi);

		gsl_vector_set(y, i, yi);
		gsl_vector_set(w, i, 1.0 / (ei*ei));
	}

		gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc(n, 2);
		if( 0!=gsl_multifit_wlinear(X, w, y, c, cov, &chisq, work) )return 1;
		gsl_multifit_linear_free(work);

#define C(i) (gsl_vector_get(c,(i)))
#define COV(i,j) (gsl_matrix_get(cov,(i),(j)))

		poly_coeff[0] = C(0);
		poly_coeff[1] = C(1);
		poly_coeff[2] = 0.0; //C(2);
		poly_coeff[5] = COV(0, 0);
		poly_coeff[6] = COV(1, 1);
		poly_coeff[7] = 0.0; //COV(2, 2);

        /* Free GSL stuff */
        gsl_matrix_free(X);
        gsl_vector_free(y);
        gsl_vector_free(w);
        gsl_vector_free(c);
        gsl_matrix_free(cov);

	return 0;
}

