#!/bin/bash

rm -f calib.txt
if [ ! -f data.m_sigma ];then
 lib/find_candidates_MAG_SIGMA_MAG.sh
fi

if [ -z $1 ] ;then
 echo "Performing manual magnitude calibration."
 echo "Please specify the comparison stars and their magnitudes!"
 ./pgfv calib
else
 # Magnitude calibration for BVR bands using USNO-B1.0
 
 BAND=$1

 echo "Performing automatic magnitude calibration using $BAND magnitudes from the USNO-B1 catalog."
 
# which vizquery &>/dev/null
# if [ $? -ne 0 ];then
#  echo " "
#  echo "ERROR: vizquery not found. :("
#  echo "Please install cdsclient ( http://cdsweb.u-strasbg.fr/vizier/doc/cdsclient.html )."
#  echo " "
#  exit
# fi
 
 util/star_catalog.sh $2
# cat vast_star_catalog.log | awk '{print $4" "$5}' | grep ":" > a.txt
 cat vast_star_catalog.log | awk '{print $4" "$5}' > a.txt

# REF_IMAGE=`grep "Ref.  image:" vast_summary.log |awk '{print $6}'`
# REF_IMAGE_CATALOG=`util/wcs_image_calibration.sh $REF_IMAGE |grep ".cat" | awk '{print $2}'`
# cat $REF_IMAGE_CATALOG | awk '{print $2"   "$3}' > a.txt

 if [ "$BAND" == "V" ];then
  # Compute V mag from B1 and R1 following John Greaves 
  # http://www.aerith.net/astro/color_conversion/JG/USNO-B1.0.html
  lib/vizquery -mime=text -source=USNO-B1 -out.max=1 -out.add=_1 -out.add=_r -out.form=mini -out=B1mag,R1mag -sort=_r -list=a.txt -c.rs=3 |grep -v \# | grep -v "_" | grep -v "\-\-\-" | while read RA DEC R M1 M2 ;do
   if [ ! -z $M1 ];then
    if [ ! -z $M2 ];then
     # Check if the stars are not too faint
     TEST_MAG=`echo "$M1" | awk '{printf "%.0f",$1}'`
     if [ $TEST_MAG -ge 18 ];then
      continue
     fi
     TEST_MAG=`echo "$M2" | awk '{printf "%.0f",$1}'`
     if [ $TEST_MAG -ge 18 ];then
      continue
     fi
     # End of test
     STRING=`grep "$RA" vast_star_catalog.log | grep "$DEC"`
     INST_MAG=`echo $STRING | awk '{print $1}'`
     INST_ERR=`echo $STRING | awk '{print $2}'`
     CAT_MAG=`echo "0.444*$M1+0.556*$M2"|bc -ql|awk '{printf "%.3f",$1}'`
     echo "$INST_MAG $CAT_MAG $INST_ERR"
    fi
   fi
   unset M1
   unset M2
  done > calib.txt
 else
  lib/vizquery -mime=text -source=USNO-B1 -out.max=1 -out.add=_1 -out.add=_r -out.form=mini -out="$BAND"1mag,"$BAND"2mag -sort=_r -list=a.txt -c.rs=3 |grep -v \# | grep -v "_" | grep -v "\-\-\-" | while read RA DEC R M1 M2 ;do
   if [ ! -z $M1 ];then
    if [ ! -z $M2 ];then
     # Check if the stars are not too faint
     TEST_MAG=`echo "$M1" | awk '{printf "%.0f",$1}'`
     if [ $TEST_MAG -ge 18 ];then
      continue
     fi
     TEST_MAG=`echo "$M2" | awk '{printf "%.0f",$1}'`
     if [ $TEST_MAG -ge 18 ];then
      continue
     fi
     # End of test
     STRING=`grep "$RA" vast_star_catalog.log | grep "$DEC"`
     INST_MAG=`echo $STRING | awk '{print $1}'`
     INST_ERR=`echo $STRING | awk '{print $2}'`
     CAT_MAG=`echo "($M1+$M2)/2.0"|bc -ql|awk '{printf "%.3f",$1}'`
     echo "$INST_MAG $CAT_MAG $INST_ERR"
    fi
   fi
   unset M1
   unset M2
  done > calib.txt
 fi
 
fi

echo "Saving a copy of calib.txt in calib.txt_backup"
cp calib.txt calib.txt_backup
util/calibrate_magnitude_scale `lib/fit_mag_calib`
lib/find_candidates_MAG_SIGMA_MAG.sh
echo "done =)"
