#!/bin/bash
#
# This small script will create a new directory named REGION_NAME and save the current VaST work to it.
# To resume work with the current field, just copy all files from the directory REGION_NAME to to the
# VaST directory and run ./find_candidates aa
#
if [ ! -z $1 ];then
 REGION_NAME="$1"
else
 echo "Region name?"
 read REGION_NAME
fi
echo -n "Creating directory $REGION_NAME ... "
mkdir "$REGION_NAME"
if [ $? == 0 ];then
 echo "Ok"
else
 echo "ERROR: Can't create directory $REGION_NAME"
 exit 1
fi

if [ ! -d "$REGION_NAME" ];then
 echo "ERROR: Can't open directory $REGION_NAME"
 exit 1
fi

for WCS_IMAGE in wcs_* ;do
 if [ -f $WCS_IMAGE ];then
  echo -n "Saving WCS-calibrated image  $WCS_IMAGE ... "
  mv $WCS_IMAGE "$REGION_NAME" && echo "Ok"
 fi
done

# For compatibility with an ancient matching technique.
# The wcs.fit file is produced by ./pgfv match
if [ -f wcs.fit ];then
 echo -n "Saving wcs.fit ... "
 mv wcs.fit "$REGION_NAME" && echo "Ok"
fi
 
if [ -f calib.txt ];then
 echo -n "Saving calib.txt ... "
 mv calib.txt "$REGION_NAME" && echo "Ok"
fi
if [ -d vast_magnitude_calibration_details_log ];then
 echo -n "Saving vast_magnitude_calibration_details_log ... "
 mv vast_magnitude_calibration_details_log/ "$REGION_NAME" && echo "Ok"
fi

echo -n "Saving" `grep "SExtractor parameter file:" vast_summary.log |awk '{print $4}'`" ... " 
cp `grep "SExtractor parameter file:" vast_summary.log |awk '{print $4}'` "$REGION_NAME" && echo "Ok"

echo -n "Saving the program source code..."
cp -r src/ "$REGION_NAME"/vast_src_"$REGION_NAME" && echo "Ok"


echo -n "Saving other files ... "
for i in out*dat candidates*lst data* vast*.log sysrem_input_star_list.lst ref_frame_sextractor.cat aavso_* image*.cat* ;do 
 if [ -f $i ];then
  mv $i "$REGION_NAME" 
 fi
done && echo -e "Ok\nAll data and parameters saved!\nTo restore them use:\n util/load.sh $REGION_NAME" && exit

echo "Something is wrong..."
