#!/bin/bash
#
# This script will search the web for known variable stars
#
echo -e "Starting $0"

# Test the command line arguments
if [ -z $2 ];then
 echo " "
 echo "ERROR: search coordinates are not given! :("
 echo " "
 echo "Usage: $0 RA DEC"
 exit
fi
RA=$1
DEC=$2

if [ -z $3 ];then
 COLOR=1
else
 COLOR=0
fi

echo "Searching databases for object $RA $DEC"

# Thest if curl is installed
CURL=`which curl`
if [ $? -ne 0 ];then
 echo " "
 echo "ERROR: curl not found. :("
 echo "No web search will be done!"
 echo " "
 exit
fi

# Querry GCVS
if [ $COLOR -eq 1 ];then
 DATABASE_NAME="\033[01;35mGCVS\033[00m"
else
 DATABASE_NAME="<font color=\"purple\">GCVS</font>"
fi
DEC_STUPID_PLUS_GCVS=`echo $DEC |awk -F: '{print $1}'`
echo $DEC_STUPID_PLUS_GCVS |grep \- &>/dev/null
if [ $? -eq 0 ];then
 DEC_STUPID_PLUS_GCVS="$DEC"
else
 DEC_STUPID_PLUS_GCVS=\%2B${DEC//+/""}
fi
DATABASE_RESULTS=`$CURL --silent --max-time 30 "http://www.sai.msu.su/gcvs/cgi-bin/co-h.cgi?coor=${RA//:/+}+${DEC_STUPID_PLUS_GCVS//:/+}&radius=60"|grep \|`
#echo "## $DATABASE_RESULTS ##"
if [ "$DATABASE_RESULTS" != "" ];then
 if [ $COLOR -eq 1 ];then
  echo -e "The object was \033[01;32mfound\033[00m in $DATABASE_NAME:  "
 else
  echo -e "The object was <font color=\"green\">found</font> in $DATABASE_NAME:  "
 fi
 DATABASE_RESULTS=`echo $DATABASE_RESULTS |awk -F"<P>" '{print $6}'`
 echo ${DATABASE_RESULTS//"</pre>"/""}
 exit # !!!
 
else
 if [ $COLOR -eq 1 ];then
  echo -e "The object was \033[01;31mnot found\033[00m in $DATABASE_NAME."
 else
  echo -e "The object was <font color=\"red\">not found</font> in $DATABASE_NAME."
 fi
fi
#exit
 
# Querry SIMBAD
if [ $COLOR -eq 1 ];then
 DATABASE_NAME="\033[01;31mSIMBAD\033[00m"
else
 DATABASE_NAME="<font color=\"maroon\">SIMBAD</font>"
fi
# Try to querry SIMBAD multiple times with different search radius 
# since the simple parser will recognize object if this is the single SIMBAD object in the field.
for SIMBAD_SEARCH_RADIUS in 0.05 1 0.02 0.5 ;do
 DATABASE_RESULTS=`$CURL --silent --max-time 30 --data "Coord=$RA%20$DEC&CooDefinedFrames=J2000&Radius=$SIMBAD_SEARCH_RADIUS&Radius.unit=arcmin" "http://simbad.u-strasbg.fr/simbad/sim-coo" |grep -v \< |grep -A3 "Basic" |tail -n3`
 if [ "$DATABASE_RESULTS" != "" ];then
  if [ $COLOR -eq 1 ];then
   echo -e "The object was \033[01;32mfound\033[00m in $DATABASE_NAME:  "
  else
   echo -e "The object was <font color=\"green\">found</font> in $DATABASE_NAME:  "
  fi
  echo $DATABASE_RESULTS
  break
 fi
done
if [ "$DATABASE_RESULTS" == "" ];then
 if [ $COLOR -eq 1 ];then
  echo -e "The object was \033[01;31mnot found\033[00m in $DATABASE_NAME."
 else
  echo -e "The object was <font color=\"red\">not found</font> in $DATABASE_NAME."
 fi
fi

#Querry VSX
if [ $COLOR -eq 1 ];then
 DATABASE_NAME="\033[01;34mVSX\033[00m"
else
 DATABASE_NAME="<font color=\"blue\">VSX</font>"
fi
$CURL  --silent --max-time 30 --data "targetcenter=$RA%20$DEC&format=s&constid=0&fieldsize=1.0&fieldunit=2&geometry=r&order=9&ql=1&filter[]=0,1,2" "http://www.aavso.org/vsx/index.php?view=results.submit1" > curlhack.html
if [ ! -s curlhack.html ] ;then echo "!!! Network error: cannot connect to VSX !!!" ;fi
DATABASE_RESULTS=`grep '\<desig' curlhack.html |awk -F\> '{print $3}'`
if [ "$DATABASE_RESULTS" != "" ];then
 if [ $COLOR -eq 1 ];then
  echo -e "The object was \033[01;32mfound\033[00m in $DATABASE_NAME:  "
 else
  echo -e "The object was <font color=\"green\">found</font> in $DATABASE_NAME:  "
 fi
 echo -e "${DATABASE_RESULTS//"</a"/     }"
else
 if [ $COLOR -eq 1 ];then
  echo -e "The object was \033[01;31mnot found\033[00m in $DATABASE_NAME."
 else
  echo -e "The object was <font color=\"red\">not found</font> in $DATABASE_NAME."
 fi
fi
rm -f curlhack.html
