#!/bin/bash
#
echo -e "Starting $0" > /dev/stderr

# Test the command line arguments
if [ -z $2 ];then
 echo " "
 echo "ERROR: search coordinates are not given! :("
 echo " "
 echo "Usage: $0 RA DEC"
 exit
fi   
RA=$1
DEC=$2

STAR_NAME=$3

R_SEARCH_ARCSEC=2.5
echo "Searching 2MASS $R_SEARCH_ARCSEC\" around $RA $DEC"
lib/vizquery -mime=text -source=2MASS  -out.max=1 -out.add=_r -out.form=mini  -sort=_r  -c="$RA $DEC" -c.rs=$R_SEARCH_ARCSEC -out=Jmag,e_Jmag,Kmag,e_Kmag  2>/dev/null |grep -v \# | grep -v "_" | grep -v "\-\-\-" |grep -v "sec"  | while read R J eJ K eK REST ;do
 if [ ! -z $R ];then
  # Compute J-K
  J_K=`echo "$J-$K" | bc -ql | awk '{printf "%.3f",$1}'`
  eJ_K=`echo "sqrt($eJ*$eJ+$eK*$eK)" | bc -ql | awk '{printf "%.3f",$1}'`
  if [ ! -z $J_K ];then
   # Guess spectral type *assuming zero extinction*
   # Based on http://adsabs.harvard.edu/abs/1988PASP..100.1134B
   SECTRAL_TYPE="B"
   TEST=`echo "$J_K > 0.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="A"
   fi
   TEST=`echo "$J_K > 0.16"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="F"
   fi
   TEST=`echo "$J_K > 0.36"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="G"
   fi
   TEST=`echo "$J_K > 0.53"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="K"
   fi
   TEST=`echo "$J_K > 0.86"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="M"
   fi
   TEST=`echo "$J_K > 1.5"|bc -ql`
   if [ $TEST -eq 1 ];then
    SECTRAL_TYPE="Very red!"
   fi
  else
   SECTRAL_TYPE="Sorry, cannot get 2MASS color"
  fi # if [ ! -z $J_K ];then
  # Print results
  echo "r=$R\" J = $J +/-$eJ  Ks = $K +/-$eK  J-Ks =  $J_K +/-$eJ_K  ($SECTRAL_TYPE)"
  echo "Spectral type is according to Bessell & Brett (1988, PASP, 100, 1134) *assuming zero extinction*."
  echo "J-Ks=$J_K+/-$eJ_K ($SECTRAL_TYPE)" > 2mass.tmp
 fi
done

R_SEARCH_ARCSEC=2.5
echo " "
echo "Searching USNO-B1.0 for the brightest objects within $R_SEARCH_ARCSEC\" around $RA $DEC"
#echo " "
lib/vizquery -mime=text -source=USNO-B1 -out.max=100 -out.add=_r -out.form=mini -sort=B2mag  -c="$RA $DEC" -c.rs=$R_SEARCH_ARCSEC -out=USNO-B1.0,RAJ2000,DEJ2000,B2mag  2>/dev/null |grep -v \# | grep -v "_" | grep -v "\-\-\-" |grep -v "sec"  |grep -v "RAJ" |grep -v "B" | while read R USNOB1 CATRA_DEG CATDEC_DEG B2 REST ;do
 # skip stars with unknown B2mag
 if [ -z $B2 ] ;then
  continue
 fi
 # skip empty lines if for whatever reason they were not caught before
 if [ ! -z $R ] ;then
  GOOD_CATALOG_POSITION=`lib/deg2hms $CATRA_DEG $CATDEC_DEG` 
  GOOD_CATALOG_NAME=$USNOB1
  echo "***************************************"
  echo "r=$R\" $USNOB1 $GOOD_CATALOG_POSITION B2=$B2"
  #echo " "
  echo "Summary string:"
  GCVS_RESULT=`lib/vizquery -mime=text -source=B/gcvs -out.max=1 -out.form=mini   -sort=_r -c="$GOOD_CATALOG_POSITION" -c.rs=5 -out=GCVS,VarType,Period 2>/dev/null  |grep -v \# | grep -v "_" | grep -v "\-\-\-" |grep -v "GCVS" |head -n2 |tail -n 1`
  GCVS_V=`echo "$GCVS_RESULT" | awk '{print $1}'`
  if [ "$GCVS_V" != "" ] ;then
   # STAR FROM GCVS
   GCVS_CONSTEL=`echo "$GCVS_RESULT" | awk '{print $2}'`
   GCVS_TYPE=`echo "$GCVS_RESULT" | awk '{print $3}'`
   GCVS_PERIOD=`echo "$GCVS_RESULT" | awk '{print $4}'`
   echo -n " $STAR_NAME | $GCVS_V $GCVS_CONSTEL | $GOOD_CATALOG_POSITION | $GCVS_TYPE (GCVS) | $GCVS_PERIOD (GCVS) | "
  else
   VSX_RESULT=`lib/vizquery -mime=text -source=B/vsx -out.max=1 -out.form=mini   -sort=_r -c="$GOOD_CATALOG_POSITION" -c.rs=5 -out=Name,Type,Period |grep -v \# | grep -v "_" | grep -v "\-\-\-" |grep -A1 Name | tail -n1`
   VSX_V=`echo "$VSX_RESULT" | awk '{print $1}'`
   if [ "$VSX_V" != "" ] ;then
    # STAR FROM VSX
    VSX_NAME=`echo "$VSX_RESULT" | awk '{print $2}'`
    VSX_TYPE=`echo "$VSX_RESULT" | awk '{print $3}'`
    VSX_PERIOD=`echo "$VSX_RESULT" | awk '{print $4}'`
    echo -n " $STAR_NAME | $VSX_V $VSX_NAME | $GOOD_CATALOG_POSITION | $VSX_TYPE (VSX) | $VSX_PERIOD (VSX) | "
   else
    # NEW var
    echo -n " $STAR_NAME | B1.0 $GOOD_CATALOG_NAME | $GOOD_CATALOG_POSITION | T | P | B2=$B2 "
   fi
  fi
  if [ -f 2mass.tmp ];then
   cat 2mass.tmp
  fi
  echo "***************************************"
  break
 fi
done
rm -f 2mass.tmp usno.tmp
